@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2013, 2015-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,21 +16,26 @@
  */
 
 #include <starpu.h>
-#include <pthread.h>
+#include <config.h>
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
+#ifdef STARPU_QUICK_CHECK
+static unsigned niter = 500;
+#elif !defined(STARPU_LONG_CHECK)
+static unsigned niter = 5000;
+#else
 static unsigned niter = 50000;
+#endif
 
-extern void cpu_codelet(void *descr[], __attribute__ ((unused)) void *_args);
+extern void cpu_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
 
 #ifdef STARPU_USE_CUDA
-extern void cuda_codelet(void *descr[], __attribute__ ((unused)) void *_args);
+extern void cuda_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
 #endif
 
 #ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-extern void opencl_codelet(void *descr[], __attribute__ ((unused)) void *_args);
+extern void opencl_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
 struct starpu_opencl_program opencl_program;
 #endif
 
@@ -39,20 +44,17 @@ int main(int argc, char **argv)
 	unsigned i;
         float foo;
 	starpu_data_handle_t float_array_handle;
-	struct starpu_codelet cl = {};
+	struct starpu_codelet cl;
 	int ret;
 
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV) goto enodev;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-#ifdef STARPU_QUICK_CHECK
-	niter /= 100;
-#endif
         if (argc == 2) niter = atoi(argv[1]);
         foo = 0.0f;
 
-	starpu_variable_data_register(&float_array_handle, 0 /* home node */,
+	starpu_variable_data_register(&float_array_handle, STARPU_MAIN_RAM /* home node */,
                                       (uintptr_t)&foo, sizeof(float));
 
 #ifdef STARPU_USE_OPENCL
@@ -60,8 +62,9 @@ int main(int argc, char **argv)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_load_opencl_from_file");
 #endif
 
-	cl.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL;
+	starpu_codelet_init(&cl);
         cl.cpu_funcs[0] = cpu_codelet;
+        cl.cpu_funcs_name[0] = "cpu_codelet";
 #ifdef STARPU_USE_CUDA
         cl.cuda_funcs[0] = cuda_codelet;
 #endif
@@ -71,11 +74,11 @@ int main(int argc, char **argv)
         cl.nbuffers = 1;
 	cl.modes[0] = STARPU_RW;
         cl.model = NULL;
+	cl.name = "variable_inc";
 
 	for (i = 0; i < niter; i++)
 	{
 		struct starpu_task *task = starpu_task_create();
-                int ret;
 
 		task->cl = &cl;
 
@@ -87,7 +90,8 @@ int main(int argc, char **argv)
 		if (STARPU_UNLIKELY(ret == -ENODEV))
 		{
 			FPRINTF(stderr, "No worker may execute this task\n");
-			exit(0);
+			starpu_data_unregister(float_array_handle);
+			goto enodev;
 		}
 	}
 
@@ -97,12 +101,12 @@ int main(int argc, char **argv)
 	starpu_data_unregister(float_array_handle);
 
 	FPRINTF(stderr, "variable -> %f\n", foo);
+	FPRINTF(stderr, "result is %scorrect\n", foo==niter?"":"IN");
 
 	starpu_shutdown();
 
-	return 0;
+	return (foo == niter) ? EXIT_SUCCESS:EXIT_FAILURE;
 
 enodev:
-	starpu_shutdown();
 	return 77;
 }
