@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,7 +24,6 @@
 #ifdef STARPU_USE_CUDA
 #include <cuda.h>
 #include <cuda_runtime.h>
-#include <cublas.h>
 #endif
 
 #include <common/blas.h>
@@ -32,7 +31,7 @@
 
 #define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 #define PRINTF(fmt, ...) do { if (!getenv("STARPU_SSILENT")) {printf(fmt, ## __VA_ARGS__); }} while(0)
-#define NMAXBLOCKS	32
+#define NMAXBLOCKS	128
 
 #define TAG11(k)	((starpu_tag_t)( (1ULL<<60) | (unsigned long long)(k)))
 #define TAG21(k,j)	((starpu_tag_t)(((3ULL<<60) | (((unsigned long long)(k))<<32)	\
@@ -113,15 +112,37 @@
 
 /* End of magma code */
 
-#ifdef STARPU_QUICK_CHECK
-static unsigned size = 320*4;
-static unsigned nblocks = 4;
-static unsigned nbigblocks = 2;
-#else
-static unsigned size = 960*16;
-static unsigned nblocks = 16;
-static unsigned nbigblocks = 8;
+static unsigned size;
+static unsigned nblocks;
+static unsigned nbigblocks;
+
+static inline void init_sizes(void) {
+	int power = starpu_cpu_worker_get_count() + 32 * starpu_cuda_worker_get_count();
+	int power_cbrt = cbrt(power);
+#ifndef STARPU_LONG_CHECK
+	power_cbrt /= 2;
 #endif
+
+	if (power_cbrt < 1)
+		power_cbrt = 1;
+
+#ifdef STARPU_QUICK_CHECK
+	if (!size)
+		size = 320*2*power_cbrt;
+	if (!nblocks)
+		nblocks = 2*power_cbrt;
+	if (!nbigblocks)
+		nbigblocks = power_cbrt;
+#else
+	if (!size)
+		size = 960*8*power_cbrt;
+	if (!nblocks)
+		nblocks = 8*power_cbrt;
+	if (!nbigblocks)
+		nbigblocks = 4*power_cbrt;
+#endif
+}
+
 static unsigned pinned = 1;
 static unsigned noprio = 0;
 static unsigned check = 0;
@@ -134,19 +155,35 @@ static unsigned with_noctxs = 0;
 static unsigned chole1 = 0;
 static unsigned chole2 = 0;
 
+struct starpu_perfmodel chol_model_11;
+struct starpu_perfmodel chol_model_21;
+struct starpu_perfmodel chol_model_22;
+
+struct starpu_codelet cl11;
+struct starpu_codelet cl21;
+struct starpu_codelet cl22;
+
 void chol_cpu_codelet_update_u11(void **, void *);
 void chol_cpu_codelet_update_u21(void **, void *);
 void chol_cpu_codelet_update_u22(void **, void *);
 
+double cpu_chol_task_11_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cpu_chol_task_21_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cpu_chol_task_22_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+
 #ifdef STARPU_USE_CUDA
 void chol_cublas_codelet_update_u11(void *descr[], void *_args);
 void chol_cublas_codelet_update_u21(void *descr[], void *_args);
 void chol_cublas_codelet_update_u22(void *descr[], void *_args);
+
+double cuda_chol_task_11_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cuda_chol_task_21_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
+double cuda_chol_task_22_cost(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl);
 #endif
 
-extern struct starpu_perfmodel chol_model_11;
-extern struct starpu_perfmodel chol_model_21;
-extern struct starpu_perfmodel chol_model_22;
+void initialize_chol_model(struct starpu_perfmodel* model, char* symbol, 
+		double (*cpu_cost_function)(struct starpu_task *, struct starpu_perfmodel_arch*, unsigned), 
+		double (*cuda_cost_function)(struct starpu_task *, struct starpu_perfmodel_arch*, unsigned));
 
 static void STARPU_ATTRIBUTE_UNUSED parse_args(int argc, char **argv)
 {
@@ -157,78 +194,70 @@ static void STARPU_ATTRIBUTE_UNUSED parse_args(int argc, char **argv)
 		{
 			with_ctxs = 1;
 			break;
-		} else
-		if (strcmp(argv[i], "-with_noctxs") == 0) 
+		}
+		else if (strcmp(argv[i], "-with_noctxs") == 0) 
 		{
 			with_noctxs = 1;
 			break;
-		} else
-		
-		if (strcmp(argv[i], "-chole1") == 0) 
+		}
+		else if (strcmp(argv[i], "-chole1") == 0) 
 		{
 			chole1 = 1;
 			break;
-		} else
-
-		if (strcmp(argv[i], "-chole2") == 0) 
+		}
+		else if (strcmp(argv[i], "-chole2") == 0) 
 		{
 			chole2 = 1;
 			break;
-		} else
-
-		if (strcmp(argv[i], "-size") == 0)
+		}
+		else if (strcmp(argv[i], "-size") == 0)
 		{
 		        char *argptr;
 			size = strtol(argv[++i], &argptr, 10);
-		} else
-
-		if (strcmp(argv[i], "-nblocks") == 0)
+		}
+		else if (strcmp(argv[i], "-nblocks") == 0)
 		{
 		        char *argptr;
 			nblocks = strtol(argv[++i], &argptr, 10);
-		} else
-
-		if (strcmp(argv[i], "-nbigblocks") == 0)
+		}
+		else if (strcmp(argv[i], "-nbigblocks") == 0)
 		{
 		        char *argptr;
 			nbigblocks = strtol(argv[++i], &argptr, 10);
-		} else
-
-		if (strcmp(argv[i], "-no-pin") == 0)
+		}
+		else if (strcmp(argv[i], "-no-pin") == 0)
 		{
 			pinned = 0;
-		} else
-
-		if (strcmp(argv[i], "-no-prio") == 0)
+		}
+		else if (strcmp(argv[i], "-no-prio") == 0)
 		{
 			noprio = 1;
-		} else
-
-		if (strcmp(argv[i], "-bound") == 0)
+		}
+		else if (strcmp(argv[i], "-commute") == 0)
+		{
+			cl22.modes[2] |= STARPU_COMMUTE;
+		}
+		else if (strcmp(argv[i], "-bound") == 0)
 		{
 			bound = 1;
-		} else
-
-		if (strcmp(argv[i], "-bound-lp") == 0)
+		}
+		else if (strcmp(argv[i], "-bound-lp") == 0)
 		{
 			bound_lp = 1;
-		} else
-
-		if (strcmp(argv[i], "-bound-mps") == 0)
+		}
+		else if (strcmp(argv[i], "-bound-mps") == 0)
 		{
 			bound_mps = 1;
-		} else
-
-		if (strcmp(argv[i], "-bound-deps") == 0)
+		}
+		else if (strcmp(argv[i], "-bound-deps") == 0)
 		{
 			bound_deps = 1;
-		} else
-
-		if (strcmp(argv[i], "-check") == 0)
+		}
+		else if (strcmp(argv[i], "-check") == 0)
 		{
 			check = 1;
-		} else
-
+		}
+		else
 		/* if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i],"--help") == 0) */
 		{
 			fprintf(stderr,"usage : %s [-size size] [-nblocks nblocks] [-no-pin] [-no-prio] [-bound] [-bound-deps] [-bound-lp] [-check]\n", argv[0]);
