@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2009-2015  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,6 +40,11 @@
 #define debug(fmt, ...)
 #endif
 
+struct starpu_perfmodel model_11;
+struct starpu_perfmodel model_12;
+struct starpu_perfmodel model_21;
+struct starpu_perfmodel model_22;
+
 static unsigned *advance_11; /* size nblocks, whether the 11 task is done */
 static unsigned *advance_12_21; /* size nblocks*nblocks */
 static unsigned *advance_22; /* array of nblocks *nblocks*nblocks */
@@ -52,6 +57,7 @@ static unsigned no_prio = 0;
 static struct starpu_codelet cl11 =
 {
 	.cpu_funcs = {dw_cpu_codelet_update_u11},
+	.cpu_funcs_name = {"dw_cpu_codelet_update_u11"},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {dw_cublas_codelet_update_u11},
 #endif
@@ -63,9 +69,11 @@ static struct starpu_codelet cl11 =
 static struct starpu_codelet cl12 =
 {
 	.cpu_funcs = {dw_cpu_codelet_update_u12},
+	.cpu_funcs_name = {"dw_cpu_codelet_update_u12"},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {dw_cublas_codelet_update_u12},
 #endif
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_RW},
 	.model = &model_12
@@ -74,24 +82,28 @@ static struct starpu_codelet cl12 =
 static struct starpu_codelet cl21 =
 {
 	.cpu_funcs = {dw_cpu_codelet_update_u21},
+	.cpu_funcs_name = {"dw_cpu_codelet_update_u21"},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {dw_cublas_codelet_update_u21},
 #endif
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_RW},
 	.model = &model_21
-}; 
+};
 
 static struct starpu_codelet cl22 =
 {
 	.cpu_funcs = {dw_cpu_codelet_update_u22},
+	.cpu_funcs_name = {"dw_cpu_codelet_update_u22"},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {dw_cublas_codelet_update_u22},
 #endif
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 3,
 	.modes = {STARPU_R, STARPU_R, STARPU_RW},
 	.model = &model_22
-}; 
+};
 
 
 
@@ -105,7 +117,7 @@ static struct starpu_codelet cl22 =
 void dw_callback_v2_codelet_update_u22(void *argcb)
 {
 	int ret;
-	cl_args *args = argcb;	
+	cl_args *args = argcb;
 
 	unsigned k = args->k;
 	unsigned i = args->i;
@@ -116,7 +128,7 @@ void dw_callback_v2_codelet_update_u22(void *argcb)
 
 	/* we did task 22k,i,j */
 	advance_22[k*nblocks*nblocks + i + j*nblocks] = DONE;
-	
+
 	if ( (i == j) && (i == k+1))
 	{
 		/* we now reduce the LU22 part (recursion appears there) */
@@ -127,9 +139,10 @@ void dw_callback_v2_codelet_update_u22(void *argcb)
 		task->callback_arg = u11arg;
 		task->cl = &cl11;
 		task->cl_arg = u11arg;
+		task->cl_arg_size = sizeof(*u11arg);
 
 		task->handles[0] = starpu_data_get_sub_data(args->dataA, 2, k+1, k+1);
-	
+
 		u11arg->dataA = args->dataA;
 		u11arg->i = k + 1;
 		u11arg->nblocks = args->nblocks;
@@ -163,6 +176,7 @@ void dw_callback_v2_codelet_update_u22(void *argcb)
 					task21->callback_arg = u21a;
 					task21->cl = &cl21;
 					task21->cl_arg = u21a;
+					task21->cl_arg_size = sizeof(*u21a);
 
 					u21a->i = k+1;
 					u21a->k = j;
@@ -199,6 +213,7 @@ void dw_callback_v2_codelet_update_u22(void *argcb)
 						task12->callback_arg = u12a;
 						task12->cl = &cl12;
 						task12->cl_arg = u12a;
+						task12->cl_arg_size = sizeof(*u12a);
 
 					u12a->i = k+1;
 					u12a->k = i;
@@ -221,7 +236,7 @@ void dw_callback_v2_codelet_update_u22(void *argcb)
 void dw_callback_v2_codelet_update_u12(void *argcb)
 {
 	int ret;
-	cl_args *args = argcb;	
+	cl_args *args = argcb;
 
 	/* now launch the update of LU22 */
 	unsigned i = args->i;
@@ -254,6 +269,7 @@ void dw_callback_v2_codelet_update_u12(void *argcb)
 				task22->callback_arg = u22a;
 				task22->cl = &cl22;
 				task22->cl_arg = u22a;
+				task22->cl_arg_size = sizeof(*u22a);
 
 				u22a->k = i;
 				u22a->i = k;
@@ -281,7 +297,7 @@ void dw_callback_v2_codelet_update_u12(void *argcb)
 void dw_callback_v2_codelet_update_u21(void *argcb)
 {
 	int ret;
-	cl_args *args = argcb;	
+	cl_args *args = argcb;
 
 	/* now launch the update of LU22 */
 	unsigned i = args->i;
@@ -314,6 +330,7 @@ void dw_callback_v2_codelet_update_u21(void *argcb)
 				task22->callback_arg = u22a;
 				task22->cl = &cl22;
 				task22->cl_arg = u22a;
+				task22->cl_arg_size = sizeof(*u22a);
 
 				u22a->k = i;
 				u22a->i = slicex;
@@ -340,7 +357,6 @@ void dw_callback_v2_codelet_update_u21(void *argcb)
 
 void dw_callback_v2_codelet_update_u11(void *argcb)
 {
-	int ret;
 	/* in case there remains work, go on */
 	cl_args *args = argcb;
 
@@ -352,13 +368,13 @@ void dw_callback_v2_codelet_update_u11(void *argcb)
 	/* we did task 11k */
 	advance_11[i] = DONE;
 
-	if (i == nblocks - 1) 
+	if (i == nblocks - 1)
 	{
 		/* we are done */
 		free(argcb);
 		return;
 	}
-	else 
+	else
 	{
 		/* put new tasks */
 		unsigned slice;
@@ -373,22 +389,25 @@ void dw_callback_v2_codelet_update_u11(void *argcb)
 			}
 			else
 			{
-				deps12 = advance_22[(i-1)*nblocks*nblocks + slice + i*nblocks];		
+				deps12 = advance_22[(i-1)*nblocks*nblocks + slice + i*nblocks];
 			}
 			if (deps12 & DONE)
 			{
 				/* we may perhaps launch the task 12i,slice */
-				 uint8_t u = STARPU_ATOMIC_OR(&advance_12_21[i*nblocks + slice], STARTED);
-				 if ((u & STARTED) == 0)
-				 {
+				uint8_t u = STARPU_ATOMIC_OR(&advance_12_21[i*nblocks + slice], STARTED);
+				if ((u & STARTED) == 0)
+				{
+					int ret;
+
 					/* we are the only one that should launch that task */
 					cl_args *u12a = malloc(sizeof(cl_args));
 
 					struct starpu_task *task12 = starpu_task_create();
-						task12->callback_func = dw_callback_v2_codelet_update_u12;
-						task12->callback_arg = u12a;
-						task12->cl = &cl12;
-						task12->cl_arg = u12a;
+					task12->callback_func = dw_callback_v2_codelet_update_u12;
+					task12->callback_arg = u12a;
+					task12->cl = &cl12;
+					task12->cl_arg = u12a;
+					task12->cl_arg_size = sizeof(*u12a);
 
 					u12a->i = i;
 					u12a->k = slice;
@@ -414,14 +433,16 @@ void dw_callback_v2_codelet_update_u11(void *argcb)
 			}
 			else
 			{
-				deps12 = advance_22[(i-1)*nblocks*nblocks + slice*nblocks + i];		
+				deps12 = advance_22[(i-1)*nblocks*nblocks + slice*nblocks + i];
 			}
 			if (deps12 & DONE)
 			{
 				/* we may perhaps launch the task 12i,slice */
-				 uint8_t u = STARPU_ATOMIC_OR(&advance_12_21[i + slice*nblocks], STARTED);
-				 if ((u & STARTED) == 0)
-				 {
+				uint8_t u = STARPU_ATOMIC_OR(&advance_12_21[i + slice*nblocks], STARTED);
+				if ((u & STARTED) == 0)
+				{
+					int ret;
+
 					/* we are the only one that should launch that task */
 					cl_args *u21a = malloc(sizeof(cl_args));
 
@@ -430,6 +451,7 @@ void dw_callback_v2_codelet_update_u11(void *argcb)
 					task21->callback_arg = u21a;
 					task21->cl = &cl21;
 					task21->cl_arg = u21a;
+					task21->cl_arg_size = sizeof(*u21a);
 
 					u21a->i = i;
 					u21a->k = slice;
@@ -455,34 +477,34 @@ void dw_callback_v2_codelet_update_u11(void *argcb)
 
 
 /*
- *	Callbacks 
+ *	Callbacks
  */
 
 
 void dw_callback_codelet_update_u11(void *argcb)
 {
-	int ret;
 	/* in case there remains work, go on */
 	cl_args *args = argcb;
 
-	if (args->i == args->nblocks - 1) 
+	if (args->i == args->nblocks - 1)
 	{
 		/* we are done */
 		free(argcb);
 		return;
 	}
-	else 
+	else
 	{
 		/* put new tasks */
 		unsigned nslices;
 		nslices = args->nblocks - 1 - args->i;
 
 		unsigned *remaining = malloc(sizeof(unsigned));
-		*remaining = 2*nslices; 
+		*remaining = 2*nslices;
 
 		unsigned slice;
 		for (slice = args->i + 1; slice < args->nblocks; slice++)
 		{
+			int ret;
 
 			/* update slice from u12 */
 			cl_args *u12a = malloc(sizeof(cl_args));
@@ -495,12 +517,14 @@ void dw_callback_codelet_update_u11(void *argcb)
 			task12->callback_arg = u12a;
 			task12->cl = &cl12;
 			task12->cl_arg = u12a;
+			task12->cl_arg_size = sizeof(*u12a);
 
 			struct starpu_task *task21 = starpu_task_create();
 			task21->callback_func = dw_callback_codelet_update_u12_21;
 			task21->callback_arg = u21a;
 			task21->cl = &cl21;
 			task21->cl_arg = u21a;
+			task21->cl_arg_size = sizeof(*u21a);
 
 			u12a->i = args->i;
 			u12a->k = slice;
@@ -525,34 +549,38 @@ void dw_callback_codelet_update_u11(void *argcb)
 			ret = starpu_task_submit(task21);
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 		}
+
+		free(remaining);
 	}
 }
 
 
 void dw_callback_codelet_update_u22(void *argcb)
 {
-	int ret;
-	cl_args *args = argcb;	
-	ANNOTATE_HAPPENS_BEFORE(args->remaining);
+	cl_args *args = argcb;
 	unsigned remaining = STARPU_ATOMIC_ADD(args->remaining, (-1));
+	ANNOTATE_HAPPENS_BEFORE(args->remaining);
 
 	if (remaining == 0)
 	{
+		int ret;
+
 		ANNOTATE_HAPPENS_AFTER(args->remaining);
 		/* all worker already used the counter */
 		free(args->remaining);
 
 		/* we now reduce the LU22 part (recursion appears there) */
 		cl_args *u11arg = malloc(sizeof(cl_args));
-	
-		struct starpu_task *task = starpu_task_create();
-			task->callback_func = dw_callback_codelet_update_u11;
-			task->callback_arg = u11arg;
-			task->cl = &cl11;
-			task->cl_arg = u11arg;
 
-			task->handles[0] = starpu_data_get_sub_data(args->dataA, 2, args->k + 1, args->k + 1);
-	
+		struct starpu_task *task = starpu_task_create();
+		task->callback_func = dw_callback_codelet_update_u11;
+		task->callback_arg = u11arg;
+		task->cl = &cl11;
+		task->cl_arg = u11arg;
+		task->cl_arg_size = sizeof(*u11arg);
+
+		task->handles[0] = starpu_data_get_sub_data(args->dataA, 2, args->k + 1, args->k + 1);
+
 		u11arg->dataA = args->dataA;
 		u11arg->i = args->k + 1;
 		u11arg->nblocks = args->nblocks;
@@ -567,8 +595,7 @@ void dw_callback_codelet_update_u22(void *argcb)
 
 void dw_callback_codelet_update_u12_21(void *argcb)
 {
-	int ret;
-	cl_args *args = argcb;	
+	cl_args *args = argcb;
 	unsigned remaining = STARPU_ATOMIC_ADD(args->remaining, -1);
 	ANNOTATE_HAPPENS_BEFORE(args->remaining);
 
@@ -588,6 +615,8 @@ void dw_callback_codelet_update_u12_21(void *argcb)
 		{
 			for (slicex = i+1; slicex < nblocks; slicex++)
 			{
+				int ret;
+
 				/* update that square matrix */
 				cl_args *u22a = malloc(sizeof(cl_args));
 
@@ -596,6 +625,7 @@ void dw_callback_codelet_update_u12_21(void *argcb)
 				task22->callback_arg = u22a;
 				task22->cl = &cl22;
 				task22->cl_arg = u22a;
+				task22->cl_arg_size = sizeof(*u22a);
 
 				u22a->k = i;
 				u22a->i = slicex;
@@ -613,13 +643,15 @@ void dw_callback_codelet_update_u12_21(void *argcb)
 				STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 			}
 		}
+
+		free(remaining);
 	}
 }
 
 
 
 /*
- *	code to bootstrap the factorization 
+ *	code to bootstrap the factorization
  */
 
 void dw_codelet_facto(starpu_data_handle_t dataA, unsigned nblocks)
@@ -633,7 +665,7 @@ void dw_codelet_facto(starpu_data_handle_t dataA, unsigned nblocks)
 
 	start = starpu_timing_now();
 
-	/* inject a new task with this codelet into the system */ 
+	/* inject a new task with this codelet into the system */
 	struct starpu_task *task = starpu_task_create();
 	task->callback_func = dw_callback_codelet_update_u11;
 	task->callback_arg = args;
@@ -679,14 +711,15 @@ void dw_codelet_facto_v2(starpu_data_handle_t dataA, unsigned nblocks)
 
 	start = starpu_timing_now();
 
-	/* inject a new task with this codelet into the system */ 
+	/* inject a new task with this codelet into the system */
 	struct starpu_task *task = starpu_task_create();
 	task->callback_func = dw_callback_v2_codelet_update_u11;
 	task->callback_arg = args;
 	task->cl = &cl11;
 	task->cl_arg = args;
+	task->cl_arg_size = sizeof(*args);
 
-	task->handles[0] = starpu_data_get_sub_data(dataA, 2, 0, 0); 
+	task->handles[0] = starpu_data_get_sub_data(dataA, 2, 0, 0);
 
 	/* schedule the codelet */
 	int ret = starpu_task_submit(task);
@@ -722,13 +755,34 @@ void initialize_system(float **A, float **B, unsigned dim, unsigned pinned)
 		exit(77);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+#ifdef STARPU_ATLAS
+	char * symbol_11 = "lu_model_11_atlas";
+	char * symbol_12 = "lu_model_12_atlas";
+	char * symbol_21 = "lu_model_21_atlas";
+	char * symbol_22 = "lu_model_22_atlas";
+#elif defined(STARPU_GOTO)
+	char * symbol_11 = "lu_model_11_goto";
+	char * symbol_12 = "lu_model_12_goto";
+	char * symbol_21 = "lu_model_21_goto";
+	char * symbol_22 = "lu_model_22_goto";
+#else
+	char * symbol_11 = "lu_model_11";
+	char * symbol_12 = "lu_model_12";
+	char * symbol_21 = "lu_model_21";
+	char * symbol_22 = "lu_model_22";
+#endif
+	initialize_lu_kernels_model(&model_11,symbol_11,task_11_cost,task_11_cost_cpu,task_11_cost_cuda);
+	initialize_lu_kernels_model(&model_12,symbol_12,task_12_cost,task_12_cost_cpu,task_12_cost_cuda);
+	initialize_lu_kernels_model(&model_21,symbol_21,task_21_cost,task_21_cost_cpu,task_21_cost_cuda);
+	initialize_lu_kernels_model(&model_22,symbol_22,task_22_cost,task_22_cost_cpu,task_22_cost_cuda);
+
 	starpu_cublas_init();
 
 	if (pinned)
 	{
 		starpu_malloc((void **)A, (size_t)dim*dim*sizeof(float));
 		starpu_malloc((void **)B, (size_t)dim*sizeof(float));
-	} 
+	}
 	else
 	{
 		*A = malloc((size_t)dim*dim*sizeof(float));
@@ -752,8 +806,8 @@ void free_system(float *A, float *B, unsigned dim, unsigned pinned)
 	}
 }
 
-void dw_factoLU(float *matA, unsigned size, 
-		unsigned ld, unsigned nblocks, 
+void dw_factoLU(float *matA, unsigned size,
+		unsigned ld, unsigned nblocks,
 		unsigned version, unsigned _no_prio)
 {
 
@@ -771,7 +825,7 @@ void dw_factoLU(float *matA, unsigned size,
 
 	/* monitor and partition the A matrix into blocks :
 	 * one block is now determined by 2 unsigned (i,j) */
-	starpu_matrix_data_register(&dataA, 0, (uintptr_t)matA, ld, 
+	starpu_matrix_data_register(&dataA, STARPU_MAIN_RAM, (uintptr_t)matA, ld,
 			size, size, sizeof(float));
 
 	struct starpu_data_filter f =
@@ -800,7 +854,7 @@ void dw_factoLU(float *matA, unsigned size,
 	}
 
 	/* gather all the data */
-	starpu_data_unpartition(dataA, 0);
+	starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 
 	starpu_data_unregister(dataA);
 
