@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2015  Université de Bordeaux
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2014-2015, 2017  Université de Bordeaux
+ * Copyright (C) 2010  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,14 +22,8 @@
 
 #define STARPU_LU(name)       starpu_dlu_##name
 
-#ifdef STARPU_HAVE_MAGMA
-#include <magmablas.h>
-#define CUBLAS_GEMM	magmablas_dgemm
-#define CUBLAS_TRSM	magmablas_dtrsm
-#else
 #define CUBLAS_GEMM	cublasDgemm
 #define CUBLAS_TRSM	cublasDtrsm
-#endif
 #define CUBLAS_SCAL	cublasDscal
 #define CUBLAS_GER	cublasDger
 #define CUBLAS_SWAP	cublasDswap
@@ -49,3 +43,5 @@
 #define PIVOT_THRESHHOLD	10e-10
 
 #define CAN_EXECUTE .can_execute = can_execute,
+
+#define ISZERO(f)	(fpclassify(f) == FP_ZERO)
