@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2015  Université de Bordeaux
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2014-2015, 2017  Université de Bordeaux
+ * Copyright (C) 2010  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,15 +22,8 @@
 
 #define STARPU_LU(name)       starpu_slu_##name
 
-#ifdef STARPU_HAVE_MAGMA
-#include <magmablas.h>
-#define CUBLAS_GEMM	magmablas_sgemm
-#define CUBLAS_TRSM	magmablas_strsm
-#else
 #define CUBLAS_GEMM	cublasSgemm
 #define CUBLAS_TRSM	cublasStrsm
-#endif
-
 #define CUBLAS_SCAL	cublasSscal
 #define CUBLAS_GER	cublasSger
 #define CUBLAS_SWAP	cublasSswap
@@ -50,3 +43,5 @@
 #define PIVOT_THRESHHOLD	10e-5
 
 #define CAN_EXECUTE
+
+#define ISZERO(f)	(fpclassify(f) == FP_ZERO)
