@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,13 +27,8 @@
 #include "xlu.h"
 #include "xlu_kernels.h"
 
-#ifdef STARPU_QUICK_CHECK
-static unsigned long size = 320*4;
-static unsigned nblocks = 4;
-#else
-static unsigned long size = 960*16;
-static unsigned nblocks = 16;
-#endif
+static unsigned long size = 0;
+static unsigned nblocks = 0;
 static unsigned check = 0;
 static unsigned pivot = 0;
 static unsigned no_stride = 0;
@@ -175,12 +170,13 @@ void copy_matrix_into_blocks(void)
 static void init_matrix(void)
 {
 	/* allocate matrix */
-	starpu_malloc((void **)&A, (size_t)size*size*sizeof(TYPE));
+	starpu_malloc_flags((void **)&A, (size_t)size*size*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 	STARPU_ASSERT(A);
 
 	starpu_srand48((long int)time(NULL));
 	/* starpu_srand48(0); */
 
+#ifndef STARPU_SIMGRID
 	/* initialize matrix content */
 	unsigned long i,j;
 	for (j = 0; j < size; j++)
@@ -193,9 +189,13 @@ static void init_matrix(void)
 			A[i + j*size] += (TYPE)(I*starpu_drand48());
 #endif
 			if (i == j)
+			{
+				A[i + j*size] += 1;
 				A[i + j*size] *= 100;
+			}
 		}
 	}
+#endif
 
 }
 
@@ -311,23 +311,38 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-#ifdef STARPU_QUICK_CHECK
-	size /= 4;
-	nblocks /= 4;
-#endif
-
-	parse_args(argc, argv);
-
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV)
 		return 77;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+	int power = starpu_cpu_worker_get_count() + 32 * starpu_cuda_worker_get_count();
+	int power_cbrt = cbrt(power);
+#ifndef STARPU_LONG_CHECK
+	power_cbrt /= 2;
+#endif
+	if (power_cbrt < 1)
+		power_cbrt = 1;
+
+#ifdef STARPU_QUICK_CHECK
+	if (!size)
+		size = 320*2*power_cbrt;
+	if (!nblocks)
+		nblocks = 2*power_cbrt;
+#else
+	if (!size)
+		size = 960*8*power_cbrt;
+	if (!nblocks)
+		nblocks = 8*power_cbrt;
+#endif
+
+	parse_args(argc, argv);
+
 	starpu_cublas_init();
 
-#ifndef STARPU_SIMGRID
 	init_matrix();
 
+#ifndef STARPU_SIMGRID
 	unsigned *ipiv = NULL;
 	if (check)
 		save_matrix();
@@ -386,7 +401,6 @@ int main(int argc, char **argv)
 
 	if (bound)
 	{
-		double min;
 		if (bounddeps)
 		{
 			FILE *f = fopen("lu.pl", "w");
@@ -408,16 +422,19 @@ int main(int argc, char **argv)
 	if (check)
 	{
 		FPRINTF(stderr, "Checking result\n");
-		if (pivot) {
+		if (pivot)
+		{
 			pivot_saved_matrix(ipiv);
-			free(ipiv);
 		}
 
 		check_result();
 	}
 #endif
 
-	starpu_free(A);
+	if (pivot)
+		free(ipiv);
+
+	starpu_free_flags(A, (size_t)size*size*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 
 	starpu_cublas_shutdown();
 
