@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux
+ * Copyright (C) 2010-2011, 2014-2015, 2017  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2015, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -32,6 +32,8 @@ static int create_task_11(starpu_data_handle_t dataA, unsigned k)
 	/* which sub-data is manipulated ? */
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 
+	task->tag_id = TAG11(k);
+
 	/* this is an important task */
 	if (!no_prio)
 		task->priority = STARPU_MAX_PRIO;
@@ -51,6 +53,8 @@ static int create_task_12(starpu_data_handle_t dataA, unsigned k, unsigned j)
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, j, k);
 
+	task->tag_id = TAG12(k,j);
+
 	if (!no_prio && (j == k+1))
 		task->priority = STARPU_MAX_PRIO;
 
@@ -70,6 +74,8 @@ static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned i)
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, k, i);
 
+	task->tag_id = TAG21(k,i);
+
 	if (!no_prio && (i == k+1))
 		task->priority = STARPU_MAX_PRIO;
 
@@ -90,6 +96,8 @@ static int create_task_22(starpu_data_handle_t dataA, unsigned k, unsigned i, un
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, j, k);
 	task->handles[2] = starpu_data_get_sub_data(dataA, 2, j, i);
 
+	task->tag_id = TAG22(k,i,j);
+
 	if (!no_prio &&  (i == k + 1) && (j == k +1) )
 		task->priority = STARPU_MAX_PRIO;
 
@@ -106,7 +114,6 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks)
 {
 	double start;
 	double end;
-	int ret;
 
 	/* create all the DAG nodes */
 	unsigned i,j,k;
@@ -118,6 +125,10 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks)
 
 	for (k = 0; k < nblocks; k++)
 	{
+		int ret;
+
+		starpu_iteration_push(k);
+
 		ret = create_task_11(dataA, k);
 		if (ret == -ENODEV) return ret;
 
@@ -128,12 +139,20 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks)
 		     ret = create_task_21(dataA, k, i);
 		     if (ret == -ENODEV) return ret;
 		}
+		starpu_data_wont_use(starpu_data_get_sub_data(dataA, 2, k, k));
 
 		for (i = k+1; i<nblocks; i++)
-		     for (j = k+1; j<nblocks; j++) {
+		     for (j = k+1; j<nblocks; j++)
+		     {
 			  ret = create_task_22(dataA, k, i, j);
 			  if (ret == -ENODEV) return ret;
 		     }
+		for (i = k+1; i<nblocks; i++)
+		{
+		    starpu_data_wont_use(starpu_data_get_sub_data(dataA, 2, k, i));
+		    starpu_data_wont_use(starpu_data_get_sub_data(dataA, 2, i, k));
+		}
+		starpu_iteration_pop();
 	}
 
 	/* stall the application until the end of computations */
@@ -170,7 +189,7 @@ int STARPU_LU(lu_decomposition)(TYPE *matA, unsigned size, unsigned ld, unsigned
 
 	/* monitor and partition the A matrix into blocks :
 	 * one block is now determined by 2 unsigned (i,j) */
-	starpu_matrix_data_register(&dataA, 0, (uintptr_t)matA, ld, size, size, sizeof(TYPE));
+	starpu_matrix_data_register(&dataA, STARPU_MAIN_RAM, (uintptr_t)matA, ld, size, size, sizeof(TYPE));
 
 	struct starpu_data_filter f =
 	{
@@ -189,7 +208,7 @@ int STARPU_LU(lu_decomposition)(TYPE *matA, unsigned size, unsigned ld, unsigned
 	int ret = dw_codelet_facto_v3(dataA, nblocks);
 
 	/* gather all the data */
-	starpu_data_unpartition(dataA, 0);
+	starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 	starpu_data_unregister(dataA);
 	return ret;
 }
