@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012-2014  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2014  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,8 +19,13 @@
 #define __STARPU_THREAD_H__
 
 #include <starpu_config.h>
+#include <starpu_util.h>
 #ifdef STARPU_SIMGRID
+#ifdef STARPU_HAVE_XBT_SYNCHRO_H
+#include <xbt/synchro.h>
+#else
 #include <xbt/synchro_core.h>
+#endif
 #ifdef STARPU_HAVE_SIMGRID_MSG_H
 #include <simgrid/msg.h>
 #else
@@ -29,6 +34,7 @@
 #elif !defined(_MSC_VER) || defined(BUILDING_STARPU)
 #include <pthread.h>
 #endif
+#include <stdint.h>
 
 #ifdef __cplusplus
 extern "C"
@@ -41,12 +47,13 @@ extern "C"
 
 #ifdef STARPU_SIMGRID
 
-typedef int starpu_pthread_t;
+typedef msg_process_t starpu_pthread_t;
 typedef int starpu_pthread_attr_t;
 
 int starpu_pthread_create_on(char *name, starpu_pthread_t *thread, const starpu_pthread_attr_t *attr, void *(*start_routine) (void *), void *arg, msg_host_t host);
 int starpu_pthread_create(starpu_pthread_t *thread, const starpu_pthread_attr_t *attr, void *(*start_routine) (void *), void *arg);
 int starpu_pthread_join(starpu_pthread_t thread, void **retval);
+int starpu_pthread_exit(void *retval) STARPU_ATTRIBUTE_NORETURN;
 int starpu_pthread_attr_init(starpu_pthread_attr_t *attr);
 int starpu_pthread_attr_destroy(starpu_pthread_attr_t *attr);
 int starpu_pthread_attr_setdetachstate(starpu_pthread_attr_t *attr, int detachstate);
@@ -59,12 +66,23 @@ typedef pthread_attr_t starpu_pthread_attr_t;
 #define starpu_pthread_create pthread_create
 #define starpu_pthread_create_on(name, thread, attr, routine, arg, where) starpu_pthread_create(thread, attr, routine, arg)
 #define starpu_pthread_join pthread_join
+#define starpu_pthread_exit pthread_exit
 #define starpu_pthread_attr_init pthread_attr_init
 #define starpu_pthread_attr_destroy pthread_attr_destroy
 #define starpu_pthread_attr_setdetachstate pthread_attr_setdetachstate
 
 #endif /* STARPU_SIMGRID, _MSC_VER */
 
+#ifdef STARPU_HAVE_PTHREAD_SETNAME_NP
+#ifdef STARPU_HAVE_DARWIN
+#define starpu_pthread_setname(name) pthread_setname_np(name)
+#else
+#define starpu_pthread_setname(name) pthread_setname_np(pthread_self(), name)
+#endif
+#else
+#define starpu_pthread_setname(name)
+#endif
+
 /*
  * Encapsulation of the pthread_mutex_* functions.
  */
@@ -80,6 +98,10 @@ int starpu_pthread_mutex_destroy(starpu_pthread_mutex_t *mutex);
 int starpu_pthread_mutex_lock(starpu_pthread_mutex_t *mutex);
 int starpu_pthread_mutex_unlock(starpu_pthread_mutex_t *mutex);
 int starpu_pthread_mutex_trylock(starpu_pthread_mutex_t *mutex);
+int starpu_pthread_mutexattr_gettype(const starpu_pthread_mutexattr_t *attr, int *type);
+int starpu_pthread_mutexattr_settype(starpu_pthread_mutexattr_t *attr, int type);
+int starpu_pthread_mutexattr_destroy(starpu_pthread_mutexattr_t *attr);
+int starpu_pthread_mutexattr_init(starpu_pthread_mutexattr_t *attr);
 
 #elif !defined(_MSC_VER) || defined(BUILDING_STARPU) /* !STARPU_SIMGRID */
 
@@ -88,14 +110,32 @@ typedef pthread_mutexattr_t starpu_pthread_mutexattr_t;
 
 #define starpu_pthread_mutex_init pthread_mutex_init
 #define starpu_pthread_mutex_destroy pthread_mutex_destroy
+#define starpu_pthread_mutexattr_gettype pthread_mutexattr_gettype
+#define starpu_pthread_mutexattr_settype pthread_mutexattr_settype
+#define starpu_pthread_mutexattr_destroy pthread_mutexattr_destroy
+#define starpu_pthread_mutexattr_init pthread_mutexattr_init
+
+#ifdef STARPU_FXT_LOCK_TRACES
+int starpu_pthread_mutex_lock(starpu_pthread_mutex_t *mutex);
+int starpu_pthread_mutex_unlock(starpu_pthread_mutex_t *mutex);
+int starpu_pthread_mutex_trylock(starpu_pthread_mutex_t *mutex);
+#else
 #define starpu_pthread_mutex_lock pthread_mutex_lock
 #define starpu_pthread_mutex_unlock pthread_mutex_unlock
 #define starpu_pthread_mutex_trylock pthread_mutex_trylock
+#endif
 
 #define STARPU_PTHREAD_MUTEX_INITIALIZER PTHREAD_MUTEX_INITIALIZER
 
 #endif /* STARPU_SIMGRID, _MSC_VER */
 
+#if !defined(_MSC_VER) || defined(BUILDING_STARPU)
+int starpu_pthread_mutex_lock_sched(starpu_pthread_mutex_t *mutex);
+int starpu_pthread_mutex_unlock_sched(starpu_pthread_mutex_t *mutex);
+int starpu_pthread_mutex_trylock_sched(starpu_pthread_mutex_t *mutex);
+void starpu_pthread_mutex_check_sched(starpu_pthread_mutex_t *mutex, char *file, int line);
+#endif
+
 /*
  * Encapsulation of the pthread_key_* functions.
  */
@@ -144,7 +184,13 @@ typedef pthread_condattr_t starpu_pthread_condattr_t;
 #define starpu_pthread_cond_init pthread_cond_init
 #define starpu_pthread_cond_signal pthread_cond_signal
 #define starpu_pthread_cond_broadcast pthread_cond_broadcast
+
+#ifdef STARPU_FXT_LOCK_TRACES
+int starpu_pthread_cond_wait(starpu_pthread_cond_t *cond, starpu_pthread_mutex_t *mutex);
+#else
 #define starpu_pthread_cond_wait pthread_cond_wait
+#endif
+
 #define starpu_pthread_cond_timedwait pthread_cond_timedwait
 #define starpu_pthread_cond_destroy pthread_cond_destroy
 
@@ -174,11 +220,20 @@ typedef pthread_rwlockattr_t starpu_pthread_rwlockattr_t;
 
 #define starpu_pthread_rwlock_init pthread_rwlock_init
 #define starpu_pthread_rwlock_destroy pthread_rwlock_destroy
+
+#ifdef STARPU_FXT_LOCK_TRACES
+int starpu_pthread_rwlock_rdlock(starpu_pthread_rwlock_t *rwlock);
+int starpu_pthread_rwlock_tryrdlock(starpu_pthread_rwlock_t *rwlock);
+int starpu_pthread_rwlock_wrlock(starpu_pthread_rwlock_t *rwlock);
+int starpu_pthread_rwlock_trywrlock(starpu_pthread_rwlock_t *rwlock);
+int starpu_pthread_rwlock_unlock(starpu_pthread_rwlock_t *rwlock);
+#else
 #define starpu_pthread_rwlock_rdlock pthread_rwlock_rdlock
 #define starpu_pthread_rwlock_tryrdlock pthread_rwlock_tryrdlock
 #define starpu_pthread_rwlock_wrlock pthread_rwlock_wrlock
 #define starpu_pthread_rwlock_trywrlock pthread_rwlock_trywrlock
 #define starpu_pthread_rwlock_unlock pthread_rwlock_unlock
+#endif
 
 #endif /* STARPU_SIMGRID, _MSC_VER */
 
@@ -188,6 +243,11 @@ typedef pthread_rwlockattr_t starpu_pthread_rwlockattr_t;
 
 #if defined(STARPU_SIMGRID) || (!defined(STARPU_HAVE_PTHREAD_BARRIER) && (!defined(_MSC_VER) || defined(BUILDING_STARPU)))
 
+#if defined(STARPU_SIMGRID) && defined(STARPU_SIMGRID_HAVE_XBT_BARRIER_INIT)
+typedef xbt_bar_t starpu_pthread_barrier_t;
+typedef int starpu_pthread_barrierattr_t;
+#define STARPU_PTHREAD_BARRIER_SERIAL_THREAD XBT_BARRIER_SERIAL_PROCESS
+#else
 typedef struct {
 	starpu_pthread_mutex_t mutex;
 	starpu_pthread_cond_t cond;
@@ -198,6 +258,7 @@ typedef struct {
 } starpu_pthread_barrier_t;
 typedef int starpu_pthread_barrierattr_t;
 #define STARPU_PTHREAD_BARRIER_SERIAL_THREAD -1
+#endif
 
 int starpu_pthread_barrier_init(starpu_pthread_barrier_t *barrier, const starpu_pthread_barrierattr_t *attr, unsigned count);
 int starpu_pthread_barrier_destroy(starpu_pthread_barrier_t *barrier);
@@ -210,11 +271,54 @@ typedef pthread_barrierattr_t starpu_pthread_barrierattr_t;
 
 #define starpu_pthread_barrier_init pthread_barrier_init
 #define starpu_pthread_barrier_destroy pthread_barrier_destroy
+
+#ifdef STARPU_FXT_LOCK_TRACES
+int starpu_pthread_barrier_wait(starpu_pthread_barrier_t *barrier);
+#else
 #define starpu_pthread_barrier_wait pthread_barrier_wait
+#endif
 #define STARPU_PTHREAD_BARRIER_SERIAL_THREAD PTHREAD_BARRIER_SERIAL_THREAD
 
 #endif /* STARPU_SIMGRID, !STARPU_HAVE_PTHREAD_BARRIER, _MSC_VER */
 
+/*
+ * Encapsulation of the pthread_spin_* functions.
+ */
+
+#if defined(STARPU_SIMGRID) || (defined(STARPU_LINUX_SYS) && defined(STARPU_HAVE_XCHG)) || !defined(STARPU_HAVE_PTHREAD_SPIN_LOCK)
+
+typedef struct
+{
+#ifdef STARPU_SIMGRID
+	int taken;
+#elif defined(STARPU_LINUX_SYS) && defined(STARPU_HAVE_XCHG)
+	unsigned taken STARPU_ATTRIBUTE_ALIGNED(16);
+#else /* we only have a trivial implementation yet ! */
+	uint32_t taken STARPU_ATTRIBUTE_ALIGNED(16);
+#endif
+} starpu_pthread_spinlock_t;
+
+int starpu_pthread_spin_init(starpu_pthread_spinlock_t *lock, int pshared);
+int starpu_pthread_spin_destroy(starpu_pthread_spinlock_t *lock);
+int starpu_pthread_spin_lock(starpu_pthread_spinlock_t *lock);
+int starpu_pthread_spin_trylock(starpu_pthread_spinlock_t *lock);
+int starpu_pthread_spin_unlock(starpu_pthread_spinlock_t *lock);
+
+#elif !defined(_MSC_VER) /* !( defined(STARPU_SIMGRID) || !defined(STARPU_HAVE_PTHREAD_SPIN_LOCK)) */
+
+typedef pthread_spinlock_t starpu_pthread_spinlock_t;
+#define starpu_pthread_spin_init pthread_spin_init
+#define starpu_pthread_spin_destroy pthread_spin_destroy
+#define starpu_pthread_spin_lock pthread_spin_lock
+#define starpu_pthread_spin_trylock pthread_spin_trylock
+#define starpu_pthread_spin_unlock pthread_spin_unlock
+
+#endif /* !( defined(STARPU_SIMGRID) || !defined(STARPU_HAVE_PTHREAD_SPIN_LOCK)) */
+
+/*
+ * Other needed pthread definitions
+ */
+
 #if defined(_MSC_VER) && !defined(BUILDING_STARPU)
 typedef void* starpu_pthread_rwlock_t;
 typedef void* starpu_pthread_mutex_t;
