@@ -28,8 +28,7 @@ static int _compute_priority(unsigned sched_ctx)
 	int worker;
 
 	struct starpu_sched_ctx_iterator it;
-	if(workers->init_iterator)
-		workers->init_iterator(workers, &it);
+	workers->init_iterator(workers, &it);
 
 	while(workers->has_next(workers, &it))
 	{
@@ -118,8 +117,7 @@ int* sc_hypervisor_get_idlest_workers(unsigned sched_ctx, int *nworkers, enum st
 	int considered = 0;
 
 	struct starpu_sched_ctx_iterator it;
-	if(workers->init_iterator)
-		workers->init_iterator(workers, &it);
+	workers->init_iterator(workers, &it);
 
 	for(index = 0; index < *nworkers; index++)
 	{
@@ -185,8 +183,7 @@ int sc_hypervisor_get_movable_nworkers(struct sc_hypervisor_policy_config *confi
 	int worker;
 
 	struct starpu_sched_ctx_iterator it;
-	if(workers->init_iterator)
-		workers->init_iterator(workers, &it);
+	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
 	{
 		worker = workers->get_next(workers, &it);
@@ -252,7 +249,7 @@ unsigned sc_hypervisor_policy_resize(unsigned sender_sched_ctx, unsigned receive
 {
 	int ret = 1;
 	if(force_resize)
-		starpu_pthread_mutex_lock(&act_hypervisor_mutex);
+		STARPU_PTHREAD_MUTEX_LOCK(&act_hypervisor_mutex);
 	else
 		ret = starpu_pthread_mutex_trylock(&act_hypervisor_mutex);
 	if(ret != EBUSY)
@@ -288,7 +285,7 @@ unsigned sc_hypervisor_policy_resize(unsigned sender_sched_ctx, unsigned receive
 				free(workers_to_move);
 			}
 		}
-		starpu_pthread_mutex_unlock(&act_hypervisor_mutex);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&act_hypervisor_mutex);
 		return 1;
 	}
 	return 0;
@@ -310,7 +307,7 @@ double sc_hypervisor_get_slowest_ctx_exec_time(void)
 	double slowest_time = 0.0;
 
 	int s;
-	struct sc_hypervisor_wrapper* sc_w;		
+	struct sc_hypervisor_wrapper* sc_w;
 	for(s = 0; s < nsched_ctxs; s++)
 	{
 		sc_w = sc_hypervisor_get_wrapper(sched_ctxs[s]);
@@ -334,14 +331,14 @@ double sc_hypervisor_get_fastest_ctx_exec_time(void)
  	double fastest_time = curr_time;
 
 	int s;
-	struct sc_hypervisor_wrapper* sc_w;		
+	struct sc_hypervisor_wrapper* sc_w;
 	for(s = 0; s < nsched_ctxs; s++)
 	{
 		sc_w = sc_hypervisor_get_wrapper(sched_ctxs[s]);
 
 		struct sc_hypervisor_policy_config *config = sc_hypervisor_get_config(sc_w->sched_ctx);
 		double elapsed_time = (config->ispeed_ctx_sample/1000000000.0)/sc_hypervisor_get_ctx_speed(sc_w);
-		
+
 		if(elapsed_time < fastest_time)
 			fastest_time = elapsed_time;
 
@@ -388,7 +385,7 @@ enum starpu_worker_archtype sc_hypervisor_get_arch_for_index(unsigned w, struct
 
 unsigned sc_hypervisor_get_index_for_arch(enum starpu_worker_archtype arch, struct types_of_workers *tw)
 {
-	
+
 	if(arch == STARPU_CPU_WORKER)
 	{
 		if(tw->ncpus != 0)
@@ -411,12 +408,15 @@ void sc_hypervisor_get_tasks_times(int nw, int nt, double times[nw][nt], int *wo
 {
         struct sc_hypervisor_policy_task_pool *tp;
         int w, t;
+	for(w = 0; w < nw; w++)
+		for(t = 0; t < nt; t++)
+			times[w][t] = NAN;
         for (w = 0; w < nw; w++)
         {
                 for (t = 0, tp = task_pools; tp; t++, tp = tp->next)
                 {
 			int worker = workers == NULL ? w : workers[w];
-                        enum starpu_perfmodel_archtype arch = starpu_worker_get_perf_archtype(worker);
+                        struct starpu_perfmodel_arch* arch = starpu_worker_get_perf_archtype(worker, STARPU_NMAX_SCHED_CTXS);
                         double length = starpu_perfmodel_history_based_expected_perf(tp->cl->model, arch, tp->footprint);
 
                         if (isnan(length))
@@ -424,7 +424,6 @@ void sc_hypervisor_get_tasks_times(int nw, int nt, double times[nw][nt], int *wo
 			else
 			{
                                 times[w][t] = (length / 1000.);
-
 				double transfer_time = 0.0;
 				unsigned worker_in_ctx = starpu_sched_ctx_contains_worker(worker, tp->sched_ctx_id);
 				enum starpu_worker_archtype arch = starpu_worker_get_type(worker);
@@ -432,20 +431,23 @@ void sc_hypervisor_get_tasks_times(int nw, int nt, double times[nw][nt], int *wo
 				{
 					if(arch == STARPU_CUDA_WORKER)
 					{
-						double transfer_speed = starpu_transfer_bandwidth(0, starpu_worker_get_memory_node(worker));
-						transfer_time +=  (tp->data_size / transfer_speed) / 1000. ;
-						double latency = starpu_transfer_latency(0, starpu_worker_get_memory_node(worker));
+						double transfer_speed = starpu_transfer_bandwidth(STARPU_MAIN_RAM, starpu_worker_get_memory_node(worker));
+						if(transfer_speed > 0.0)
+							transfer_time +=  (tp->data_size / transfer_speed) / 1000. ;
+
+						double latency = starpu_transfer_latency(STARPU_MAIN_RAM, starpu_worker_get_memory_node(worker));
 						transfer_time += latency/1000.;
-						
-						
+//						transfer_time *=4;
 					}
 					else if(arch == STARPU_CPU_WORKER)
 					{
 						if(!starpu_sched_ctx_contains_type_of_worker(arch, tp->sched_ctx_id))
 						{
-							double transfer_speed = starpu_transfer_bandwidth(starpu_worker_get_memory_node(worker), 0);
-							transfer_time += (tp->data_size / transfer_speed) / 1000. ;
-							double latency = starpu_transfer_latency(starpu_worker_get_memory_node(worker), 0);
+							double transfer_speed = starpu_transfer_bandwidth(starpu_worker_get_memory_node(worker), STARPU_MAIN_RAM);
+							if(transfer_speed > 0.0)
+								transfer_time += (tp->data_size / transfer_speed) / 1000. ;
+
+							double latency = starpu_transfer_latency(starpu_worker_get_memory_node(worker), STARPU_MAIN_RAM);
 							transfer_time += latency / 1000.;
 						}
 					}
@@ -465,10 +467,9 @@ unsigned sc_hypervisor_check_idle(unsigned sched_ctx, int worker)
 	struct sc_hypervisor_policy_config *config = sc_w->config;
 	if(config != NULL)
 	{
-		printf("w%d/ctx%d: current idle %lf max_idle %lf\n", worker, sched_ctx, sc_w->idle_time[worker], config->max_idle[worker]);
 		if(sc_w->idle_time[worker] > config->max_idle[worker])
 		{
-//			sc_w->current_idle_time[worker] = 0.0;
+//			printf("w%d/ctx%d: current idle %lf  max_idle %lf\n", worker, sched_ctx, sc_w->idle_time[worker], config->max_idle[worker]);
 			return 1;
 		}
 	}
@@ -477,22 +478,22 @@ unsigned sc_hypervisor_check_idle(unsigned sched_ctx, int worker)
 }
 
 /* check if there is a big speed gap between the contexts */
-unsigned sc_hypervisor_check_speed_gap_btw_ctxs(void)
+unsigned sc_hypervisor_check_speed_gap_btw_ctxs(unsigned *sched_ctxs_in, int ns_in, int *workers_in, int nworkers_in)
 {
-	unsigned *sched_ctxs = sc_hypervisor_get_sched_ctxs();
-	int ns = sc_hypervisor_get_nsched_ctxs();
-	int *workers = NULL;
-	int nworkers = starpu_worker_get_count();
+	unsigned *sched_ctxs = sched_ctxs_in == NULL ? sc_hypervisor_get_sched_ctxs() : sched_ctxs_in;
+	int ns = ns_in == -1 ? sc_hypervisor_get_nsched_ctxs() : ns_in;
+	int *workers = workers_in;
+	int nworkers = nworkers_in == -1 ? starpu_worker_get_count() : nworkers_in;
 	int i = 0, j = 0;
 	struct sc_hypervisor_wrapper* sc_w;
 	struct sc_hypervisor_wrapper* other_sc_w;
 
-	
+
 	double optimal_v[ns];
 	unsigned has_opt_v = 1;
 	for(i = 0; i < ns; i++)
 	{
-		optimal_v[i] = _get_optimal_v(i);
+		optimal_v[i] = _get_optimal_v(sched_ctxs[i]);
 		if(optimal_v[i] == 0.0)
 		{
 			has_opt_v = 0;
@@ -509,10 +510,10 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(void)
 		int total_nw[nw];
 		sc_hypervisor_group_workers_by_type(tw, total_nw);
 
-		double vmax = sc_hypervisor_lp_get_nworkers_per_ctx(ns, nw, nworkers_per_ctx, total_nw, tw);
+		double vmax = sc_hypervisor_lp_get_nworkers_per_ctx(ns, nw, nworkers_per_ctx, total_nw, tw, sched_ctxs);
 
-		
-		if(vmax != 0.0)
+
+//		if(vmax != 0.0)
 		{
 			for(i = 0; i < ns; i++)
 			{
@@ -523,23 +524,23 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(void)
 				for(w = 0; w < nw; w++)
 				{
 					v[w] = sc_hypervisor_get_speed(sc_w, sc_hypervisor_get_arch_for_index(w, tw));
-					
-					optimal_v[i] += nworkers_per_ctx[i][w]*v[w];
+					optimal_v[i] += nworkers_per_ctx[i][w] == -1.0 ? 0.0 : nworkers_per_ctx[i][w]*v[w];
 				}
-				_set_optimal_v(i, optimal_v[i]);
+				_set_optimal_v(sched_ctxs[i], optimal_v[i]);
 			}
 			has_opt_v = 1;
 		}
+		free(tw);
 	}
 
-/* if we have an optimal speed for each type of worker compare the monitored one with the 
+/* if we have an optimal speed for each type of worker compare the monitored one with the
    theoretical one */
 	if(has_opt_v)
 	{
 		for(i = 0; i < ns; i++)
 		{
 			sc_w = sc_hypervisor_get_wrapper(sched_ctxs[i]);
-			
+
 			double ctx_v = sc_hypervisor_get_ctx_speed(sc_w);
 			if(ctx_v == -1.0)
 				return 0;
@@ -548,10 +549,14 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(void)
 		for(i = 0; i < ns; i++)
 		{
 			sc_w = sc_hypervisor_get_wrapper(sched_ctxs[i]);
-			
+
 			double ctx_v = sc_hypervisor_get_ctx_speed(sc_w);
-			if(ctx_v != -1.0 && ((ctx_v < 0.8*optimal_v[i]) || ctx_v > 1.2*optimal_v[i])) 
+			ctx_v = ctx_v < 0.01 ? 0.0 : ctx_v;
+			double max_vel = _get_max_speed_gap();
+			if(ctx_v != -1.0 && ((ctx_v < (1-max_vel)*optimal_v[i]) || ctx_v > (1+max_vel)*optimal_v[i]))
+			{
 				return 1;
+			}
 		}
 	}
 	else /* if we have not been able to compute a theoretical speed consider the env variable
@@ -571,7 +576,7 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(void)
 						unsigned nworkers = starpu_sched_ctx_get_nworkers(sched_ctxs[j]);
 						if(nworkers == 0)
 							return 1;
-						
+
 						other_sc_w = sc_hypervisor_get_wrapper(sched_ctxs[j]);
 						double other_ctx_v = sc_hypervisor_get_ctx_speed(other_sc_w);
 						if(other_ctx_v != -1.0)
@@ -584,12 +589,20 @@ unsigned sc_hypervisor_check_speed_gap_btw_ctxs(void)
 					}
 				}
 			}
-			
+
 		}
 	}
 	return 0;
 }
 
+unsigned sc_hypervisor_check_speed_gap_btw_ctxs_on_level(int level, int *workers_in, int nworkers_in, unsigned father_sched_ctx_id, unsigned **sched_ctxs, int *nsched_ctxs)
+{
+	sc_hypervisor_get_ctxs_on_level(sched_ctxs, nsched_ctxs, level, father_sched_ctx_id);
+
+	if(*nsched_ctxs  > 0)
+		return sc_hypervisor_check_speed_gap_btw_ctxs(*sched_ctxs, *nsched_ctxs, workers_in, nworkers_in);
+	return 0;
+}
 
 unsigned sc_hypervisor_criteria_fulfilled(unsigned sched_ctx, int worker)
 {
@@ -599,7 +612,7 @@ unsigned sc_hypervisor_criteria_fulfilled(unsigned sched_ctx, int worker)
 		if(criteria == SC_IDLE)
 			return sc_hypervisor_check_idle(sched_ctx, worker);
 		else
-			return sc_hypervisor_check_speed_gap_btw_ctxs();
+			return sc_hypervisor_check_speed_gap_btw_ctxs(NULL, -1, NULL, -1);
 	}
 	else
 		return 0;
