@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2016, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,18 +17,23 @@
 
 #include "socl.h"
 
-
 static void soclCreateKernel_task(void *data) {
    struct _cl_kernel *k = (struct _cl_kernel *)data;
 
    int range = starpu_worker_get_range();
    cl_int err;
 
-   DEBUG_MSG("[Device %d] Creating kernel...\n", starpu_worker_get_id());
+   if (k->program->cl_programs[range] == NULL) {
+      k->errcodes[range] = CL_SUCCESS;
+      DEBUG_MSG("[Device %u] Kernel creation skipped: program has not been built for this device.\n", starpu_worker_get_id_check());
+      return;
+   }
+
+   DEBUG_MSG("[Device %u] Creating kernel...\n", starpu_worker_get_id_check());
    k->cl_kernels[range] = clCreateKernel(k->program->cl_programs[range], k->kernel_name, &err);
    if (err != CL_SUCCESS) {
       k->errcodes[range] = err;
-      ERROR_STOP("[Device %d] Unable to create kernel. Aborting.\n", starpu_worker_get_id());
+      ERROR_STOP("[Device %u] Unable to create kernel. Error %d. Aborting.\n", starpu_worker_get_id_check(), err);
       return;
    }
 
@@ -55,30 +61,18 @@ static void soclCreateKernel_task(void *data) {
    }
 }
 
-static void rk_task(void *data) {
-   cl_kernel k = (cl_kernel)data;
-
-   int range = starpu_worker_get_range();
-
-   cl_int err = clReleaseKernel(k->cl_kernels[range]);
-   if (err != CL_SUCCESS)
-      DEBUG_CL("clReleaseKernel", err);
-}
-
 static void release_callback_kernel(void * e) {
   cl_kernel kernel = (cl_kernel)e;
 
   //Free args
-  unsigned int j;
-  for (j=0; j<kernel->num_args; j++) {
-    switch (kernel->arg_type[j]) {
+  unsigned int i;
+  for (i=0; i<kernel->num_args; i++) {
+    switch (kernel->arg_type[i]) {
       case Null:
-        break;
       case Buffer:
-        gc_entity_unstore((cl_mem*)&kernel->arg_value[j]);
         break;
       case Immediate:
-        free(kernel->arg_value[j]);
+        free(kernel->arg_value[i]);
         break;
     }
   }
@@ -90,11 +84,23 @@ static void release_callback_kernel(void * e) {
     free(kernel->arg_type);
 
   //Release real kernels...
-  starpu_execute_on_each_worker(rk_task, kernel, STARPU_OPENCL);
+  for (i=0; i<socl_device_count; i++) {
+     if (kernel->cl_kernels[i] != NULL) {
+        cl_int err = clReleaseKernel(kernel->cl_kernels[i]);
+        if (err != CL_SUCCESS)
+           DEBUG_CL("clReleaseKernel", err);
+     }
+  }
+
+  //Release perfmodel
+  //FIXME: we cannot release performance models before StarPU shutdown as it
+  //will use them to store kernel execution times
+
+  //free(kernel->perfmodel);
+  //free(kernel->kernel_name);
 
   gc_entity_unstore(&kernel->program);
 
-  free(kernel->kernel_name);
   free(kernel->cl_kernels);
   free(kernel->errcodes);
 }
@@ -105,7 +111,6 @@ soclCreateKernel(cl_program    program,
                cl_int *        errcode_ret) CL_API_SUFFIX__VERSION_1_0
 {
    cl_kernel k;
-   int device_count;
 
    if (program == NULL) {
       if (errcode_ret != NULL)
@@ -116,7 +121,7 @@ soclCreateKernel(cl_program    program,
    //TODO: check programs (see opencl specs)
 
    /* Create Kernel structure */
-   k = (cl_kernel)gc_entity_alloc(sizeof(struct _cl_kernel), release_callback_kernel);
+   k = (cl_kernel)gc_entity_alloc(sizeof(struct _cl_kernel), release_callback_kernel, "kernel");
    if (k == NULL) {
       if (errcode_ret != NULL)
          *errcode_ret = CL_OUT_OF_HOST_MEMORY;
@@ -125,34 +130,46 @@ soclCreateKernel(cl_program    program,
    
    gc_entity_store(&k->program, program);
    k->kernel_name = strdup(kernel_name);
+
+   k->perfmodel = malloc(sizeof(struct starpu_perfmodel));
+   memset(k->perfmodel, 0, sizeof(struct starpu_perfmodel));
+   k->perfmodel->type = STARPU_HISTORY_BASED;
+   k->perfmodel->symbol = k->kernel_name;
+
    k->num_args = 0;
    k->arg_value = NULL;
    k->arg_size = NULL;
 
+   k->split_func = NULL;
+   k->split_space = 0;
+   k->split_data = NULL;
+   k->split_perfs = NULL;
+   starpu_pthread_mutex_init(&k->split_lock, NULL);
+
    #ifdef DEBUG
    static int id = 0;
    k->id = id++;
    #endif
    
-   device_count = starpu_opencl_worker_get_count();
-   k->cl_kernels = (cl_kernel*)malloc(device_count * sizeof(cl_kernel));
-   k->errcodes = (cl_int*)malloc(device_count * sizeof(cl_int));
+   k->cl_kernels = (cl_kernel*)malloc(socl_device_count * sizeof(cl_kernel));
+   k->errcodes = (cl_int*)malloc(socl_device_count * sizeof(cl_int));
+
    {
-      int i;
-      for (i=0; i<device_count; i++) {
+      unsigned int i;
+      for (i=0; i<socl_device_count; i++) {
          k->cl_kernels[i] = NULL;
          k->errcodes[i] = -9999;
       }
    }
 
    /* Create kernel on each device */
-   DEBUG_MSG("[Kernel %d] Create %d kernels (name \"%s\")\n", k->id, starpu_opencl_worker_get_count(), kernel_name);
-   starpu_execute_on_each_worker(soclCreateKernel_task, k, STARPU_OPENCL);
+   DEBUG_MSG("[Kernel %d] Create %u kernels (name \"%s\")\n", k->id, socl_device_count, kernel_name);
+   starpu_execute_on_each_worker_ex(soclCreateKernel_task, k, STARPU_OPENCL, "SOCL_CREATE_KERNEL");
 
    if (errcode_ret != NULL) {
-      int i;
+      unsigned int i;
       *errcode_ret = CL_SUCCESS;
-      for (i=0; i<device_count; i++) {
+      for (i=0; i<socl_device_count; i++) {
          switch (k->errcodes[i]) {
             #define CASE_RET(e) case e: *errcode_ret = e; return k;
             CASE_RET(CL_INVALID_PROGRAM)
@@ -165,6 +182,11 @@ soclCreateKernel(cl_program    program,
             #undef CASE_RET
          }
       }
+
+      if (k->num_args == 666) {
+         *errcode_ret = CL_INVALID_PROGRAM_EXECUTABLE;
+         return k;
+      }
    }
 
    return k;
