@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2014  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,12 +36,16 @@
 #endif
 
 static char _STARPU_PROF_FILE_USER[128];
-static int _starpu_fxt_started = 0;
+int _starpu_fxt_started = 0;
+starpu_pthread_mutex_t _starpu_fxt_started_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
+starpu_pthread_cond_t _starpu_fxt_started_cond = STARPU_PTHREAD_COND_INITIALIZER;
 
 static int _starpu_written = 0;
 
 static int _starpu_id;
 
+static unsigned int initial_key_mask = FUT_KEYMASKALL;
+
 #ifdef STARPU_SIMGRID
 /* Give virtual time to FxT */
 uint64_t fut_getstamp(void)
@@ -52,6 +56,10 @@ uint64_t fut_getstamp(void)
 
 long _starpu_gettid(void)
 {
+	/* TODO: test at configure whether __thread is available, and use that
+	 * to cache the value.
+	 * Don't use the TSD, this is getting called before we would have the
+	 * time to allocate it.  */
 #ifdef STARPU_SIMGRID
 	return (uintptr_t) MSG_process_self();
 #else
@@ -69,19 +77,14 @@ long _starpu_gettid(void)
 #endif
 }
 
-static void _starpu_profile_set_tracefile(void *last, ...)
+static void _starpu_profile_set_tracefile(void)
 {
-	va_list vl;
 	char *user;
 
 	char *fxt_prefix = starpu_getenv("STARPU_FXT_PREFIX");
 	if (!fxt_prefix)
 	     fxt_prefix = "/tmp/";
 
-	va_start(vl, last);
-	vsprintf(_STARPU_PROF_FILE_USER, fxt_prefix, vl);
-	va_end(vl);
-
 	user = starpu_getenv("USER");
 	if (!user)
 		user = "";
@@ -89,20 +92,28 @@ static void _starpu_profile_set_tracefile(void *last, ...)
 	char suffix[128];
 	snprintf(suffix, 128, "prof_file_%s_%d", user, _starpu_id);
 
-	strcat(_STARPU_PROF_FILE_USER, suffix);
+	snprintf(_STARPU_PROF_FILE_USER, sizeof(_STARPU_PROF_FILE_USER), "%s%s", fxt_prefix, suffix);
 }
 
 void starpu_profiling_set_id(int new_id)
 {
 	_STARPU_DEBUG("Set id to <%d>\n", new_id);
 	_starpu_id = new_id;
-	_starpu_profile_set_tracefile(NULL);
+	_starpu_profile_set_tracefile();
 
 #ifdef HAVE_FUT_SET_FILENAME
 	fut_set_filename(_STARPU_PROF_FILE_USER);
 #endif
 }
 
+void starpu_fxt_autostart_profiling(int autostart)
+{
+	if (autostart)
+		initial_key_mask = FUT_KEYMASKALL;
+	else
+		initial_key_mask = FUT_KEYMASK0;
+}
+
 void starpu_fxt_start_profiling()
 {
 	unsigned threadid = _starpu_gettid();
@@ -117,16 +128,19 @@ void starpu_fxt_stop_profiling()
 	fut_keychange(FUT_DISABLE, FUT_KEYMASKALL, threadid);
 }
 
-void _starpu_init_fxt_profiling(unsigned trace_buffer_size)
+void _starpu_fxt_init_profiling(unsigned trace_buffer_size)
 {
 	unsigned threadid;
 
-	if (!_starpu_fxt_started)
-	{
-		_starpu_fxt_started = 1;
-		_starpu_written = 0;
-		_starpu_profile_set_tracefile(NULL);
-	}
+	if (!starpu_get_env_number_default("STARPU_FXT_TRACE", 1))
+		return;
+
+	STARPU_PTHREAD_MUTEX_LOCK(&_starpu_fxt_started_mutex);
+	STARPU_ASSERT(!_starpu_fxt_started);
+
+	_starpu_fxt_started = 1;
+	_starpu_written = 0;
+	_starpu_profile_set_tracefile();
 
 #ifdef HAVE_FUT_SET_FILENAME
 	fut_set_filename(_STARPU_PROF_FILE_USER);
@@ -144,15 +158,14 @@ void _starpu_init_fxt_profiling(unsigned trace_buffer_size)
 
 	atexit(_starpu_stop_fxt_profiling);
 
-	unsigned int key_mask = FUT_KEYMASKALL;
-
-	if (fut_setup(trace_buffer_size / sizeof(unsigned long), key_mask, threadid) < 0)
+	if (fut_setup(trace_buffer_size / sizeof(unsigned long), initial_key_mask, threadid) < 0)
 	{
 		perror("fut_setup");
 		STARPU_ABORT();
 	}
 
-	fut_keychange(FUT_ENABLE, key_mask, threadid);
+	STARPU_PTHREAD_COND_BROADCAST(&_starpu_fxt_started_cond);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&_starpu_fxt_started_mutex);
 
 	return;
 }
@@ -176,12 +189,14 @@ static void _starpu_generate_paje_trace(char *input_fxt_filename, char *output_p
 
 void _starpu_stop_fxt_profiling(void)
 {
+	if (!_starpu_fxt_started)
+		return;
 	if (!_starpu_written)
 	{
 #ifdef STARPU_VERBOSE
 	        char hostname[128];
 		gethostname(hostname, 128);
-		fprintf(stderr, "Writing FxT traces into file %s:%s\n", hostname, _STARPU_PROF_FILE_USER);
+		_STARPU_MSG("Writing FxT traces into file %s:%s\n", hostname, _STARPU_PROF_FILE_USER);
 #endif
 		fut_endup(_STARPU_PROF_FILE_USER);
 
@@ -195,7 +210,7 @@ void _starpu_stop_fxt_profiling(void)
 		{
 			/* Something went wrong with the FxT trace (eg. there
 			 * was too many events) */
-			fprintf(stderr, "Warning: the FxT trace could not be generated properly\n");
+			_STARPU_MSG("Warning: the FxT trace could not be generated properly\n");
 		}
 
 		_starpu_written = 1;
@@ -210,6 +225,10 @@ void _starpu_fxt_register_thread(unsigned cpuid)
 
 #else // STARPU_USE_FXT
 
+void starpu_fxt_autostart_profiling(int autostart STARPU_ATTRIBUTE_UNUSED)
+{
+}
+
 void starpu_fxt_start_profiling()
 {
 }
@@ -220,9 +239,16 @@ void starpu_fxt_stop_profiling()
 
 #endif // STARPU_USE_FXT
 
-void starpu_trace_user_event(unsigned long code STARPU_ATTRIBUTE_UNUSED)
+void starpu_fxt_trace_user_event(unsigned long code STARPU_ATTRIBUTE_UNUSED)
 {
 #ifdef STARPU_USE_FXT
 	_STARPU_TRACE_USER_EVENT(code);
 #endif
 }
+
+void starpu_fxt_trace_user_event_string(const char *s STARPU_ATTRIBUTE_UNUSED)
+{
+#ifdef STARPU_USE_FXT
+	_STARPU_TRACE_EVENT(s);
+#endif
+}
