@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012  Université de Bordeaux
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2015  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2014, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -67,17 +67,22 @@ int starpu_combined_worker_assign_workerid(int nworkers, int workerid_array[])
 	{
 		int id = workerid_array[i];
 
-		/* We only combine CPUs */
-		STARPU_ASSERT(config->workers[id].perf_arch == STARPU_CPU_DEFAULT);
-		STARPU_ASSERT(config->workers[id].worker_mask == STARPU_CPU);
-
 		/* We only combine valid "basic" workers */
 		if ((id < 0) || (id >= basic_worker_count))
 			return -EINVAL;
+
+#ifdef STARPU_USE_MIC
+		STARPU_ASSERT(config->workers[id].arch == STARPU_CPU_WORKER || config->workers[id].arch == STARPU_MIC_WORKER);
+		STARPU_ASSERT(config->workers[id].worker_mask == STARPU_CPU || config->workers[id].worker_mask == STARPU_MIC);
+#else/* STARPU_USE_MIC */
+		/* We only combine CPUs */
+		STARPU_ASSERT(config->workers[id].arch == STARPU_CPU_WORKER);
+		STARPU_ASSERT(config->workers[id].worker_mask == STARPU_CPU);
+#endif /* STARPU_USE_MIC */
 	}
 
 	/* Get an id for that combined worker. Note that this is not thread
-	 * safe because thhis method should only be called when the scheduler
+	 * safe because this method should only be called when the scheduler
 	 * is being initialized. */
 	new_workerid = basic_worker_count + combined_worker_id;
 	config->topology.ncombinedworkers++;
@@ -90,13 +95,24 @@ int starpu_combined_worker_assign_workerid(int nworkers, int workerid_array[])
 	}
 	fprintf(stderr, "into worker %d\n", new_workerid);
 #endif
+	for(i = 0; i < nworkers; i++)
+		_starpu_get_worker_struct(workerid_array[i])->combined_workerid = new_workerid;
 
 	struct _starpu_combined_worker *combined_worker =
 		&config->combined_workers[combined_worker_id];
 
 	combined_worker->worker_size = nworkers;
-	combined_worker->perf_arch = (enum starpu_perfmodel_archtype) (STARPU_CPU_DEFAULT + nworkers - 1);
-	combined_worker->worker_mask = STARPU_CPU;
+	_STARPU_MALLOC(combined_worker->perf_arch.devices, sizeof(struct starpu_perfmodel_device));
+	combined_worker->perf_arch.ndevices = 1;
+	combined_worker->perf_arch.devices[0].type = config->workers[workerid_array[0]].perf_arch.devices[0].type;
+	combined_worker->perf_arch.devices[0].devid = config->workers[workerid_array[0]].perf_arch.devices[0].devid;
+	combined_worker->perf_arch.devices[0].ncores = nworkers;
+	combined_worker->worker_mask = config->workers[workerid_array[0]].worker_mask;
+
+#ifdef STARPU_USE_MP
+	combined_worker->count = nworkers -1;
+	STARPU_PTHREAD_MUTEX_INIT(&combined_worker->count_mutex,NULL);
+#endif
 
 	/* We assume that the memory node should either be that of the first
 	 * entry, and it is very likely that every worker in the combination
@@ -125,12 +141,12 @@ int starpu_combined_worker_assign_workerid(int nworkers, int workerid_array[])
 #ifdef CPU_OR
 		CPU_OR(&combined_worker->cpu_set,
 			&combined_worker->cpu_set,
-			&config->workers[id].initial_cpu_set);
+			&config->workers[id].cpu_set);
 #else
 		int j;
 		for (j = 0; j < CPU_SETSIZE; j++)
 		{
-			if (CPU_ISSET(j, &config->workers[id].initial_cpu_set))
+			if (CPU_ISSET(j, &config->workers[id].cpu_set))
 				CPU_SET(j, &combined_worker->cpu_set);
 		}
 #endif
@@ -139,7 +155,7 @@ int starpu_combined_worker_assign_workerid(int nworkers, int workerid_array[])
 #ifdef STARPU_HAVE_HWLOC
 		hwloc_bitmap_or(combined_worker->hwloc_cpu_set,
 				combined_worker->hwloc_cpu_set,
-				config->workers[id].initial_hwloc_cpu_set);
+				config->workers[id].hwloc_cpu_set);
 #endif
 #endif
 	}
@@ -162,4 +178,3 @@ int starpu_combined_worker_get_description(int workerid, int *worker_size, int *
 
 	return 0;
 }
-
