@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012, 2014-2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
- * Copyright (C) 2012 inria
+ * Copyright (C) 2010-2012, 2014-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2017  CNRS
+ * Copyright (C) 2012 INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -82,8 +82,7 @@ int _starpu_add_successor_to_cg_list(struct _starpu_cg_list *successors, struct
 		else
 			successors->succ_list_size = 4;
 
-		successors->succ = (struct _starpu_cg **) realloc(successors->succ,
-			successors->succ_list_size*sizeof(struct _starpu_cg *));
+		_STARPU_REALLOC(successors->succ, successors->succ_list_size*sizeof(struct _starpu_cg *));
 	}
 #else
 	STARPU_ASSERT(index < STARPU_NMAXDEPS);
@@ -94,22 +93,83 @@ int _starpu_add_successor_to_cg_list(struct _starpu_cg_list *successors, struct
 	return ret;
 }
 
+int _starpu_list_task_successors_in_cg_list(struct _starpu_cg_list *successors, unsigned ndeps, struct starpu_task *task_array[])
+{
+	unsigned i;
+	unsigned n = 0;
+	_starpu_spin_lock(&successors->lock);
+	for (i = 0; i < successors->nsuccs; i++)
+	{
+		struct _starpu_cg *cg = successors->succ[i];
+		if (cg->cg_type != STARPU_CG_TASK)
+			continue;
+		if (n < ndeps)
+		{
+			task_array[n] = cg->succ.job->task;
+			n++;
+		}
+	}
+	_starpu_spin_unlock(&successors->lock);
+	return n;
+}
+
+int _starpu_list_task_scheduled_successors_in_cg_list(struct _starpu_cg_list *successors, unsigned ndeps, struct starpu_task *task_array[])
+{
+	unsigned i;
+	unsigned n = 0;
+	_starpu_spin_lock(&successors->lock);
+	for (i = 0; i < successors->nsuccs; i++)
+	{
+		struct _starpu_cg *cg = successors->succ[i];
+		if (cg->cg_type != STARPU_CG_TASK)
+			continue;
+		if (n < ndeps)
+		{
+			struct starpu_task *task = cg->succ.job->task;
+			if (task->cl == NULL || task->cl->where == STARPU_NOWHERE
+					|| task->execute_on_a_specific_worker)
+				/* will not be scheduled */
+				continue;
+			task_array[n] = task;
+			n++;
+		}
+	}
+	_starpu_spin_unlock(&successors->lock);
+	return n;
+}
+
+int _starpu_list_tag_successors_in_cg_list(struct _starpu_cg_list *successors, unsigned ndeps, starpu_tag_t tag_array[])
+{
+	unsigned i;
+	unsigned n = 0;
+	_starpu_spin_lock(&successors->lock);
+	for (i = 0; i < successors->nsuccs; i++)
+	{
+		struct _starpu_cg *cg = successors->succ[i];
+		if (cg->cg_type != STARPU_CG_TAG)
+			continue;
+		if (n < ndeps)
+		{
+			tag_array[n] = cg->succ.tag->id;
+			n++;
+		}
+	}
+	_starpu_spin_unlock(&successors->lock);
+	return n;
+}
+
 /* Note: in case of a tag, it must be already locked */
 void _starpu_notify_cg(struct _starpu_cg *cg)
 {
 	STARPU_ASSERT(cg);
-	ANNOTATE_HAPPENS_BEFORE(&cg->remaining);
 	unsigned remaining = STARPU_ATOMIC_ADD(&cg->remaining, -1);
+	ANNOTATE_HAPPENS_BEFORE(&cg->remaining);
 
 	if (remaining == 0)
 	{
 		ANNOTATE_HAPPENS_AFTER(&cg->remaining);
 		cg->remaining = cg->ntags;
 
-		struct _starpu_tag *tag;
-		struct _starpu_cg_list *tag_successors, *job_successors;
-		struct _starpu_job *j;
-
 		/* the group is now completed */
 		switch (cg->cg_type)
 		{
@@ -126,6 +186,9 @@ void _starpu_notify_cg(struct _starpu_cg *cg)
 
 			case STARPU_CG_TAG:
 			{
+				struct _starpu_cg_list *tag_successors;
+				struct _starpu_tag *tag;
+
 				tag = cg->succ.tag;
 				tag_successors = &tag->tag_successors;
 
@@ -145,6 +208,9 @@ void _starpu_notify_cg(struct _starpu_cg *cg)
 
  		        case STARPU_CG_TASK:
 			{
+				struct _starpu_cg_list *job_successors;
+				struct _starpu_job *j;
+
 				j = cg->succ.job;
 
 				STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
@@ -165,7 +231,8 @@ void _starpu_notify_cg(struct _starpu_cg *cg)
 					/* That task has already passed tag checks,
 					 * do not do them again since the tag has been cleared! */
 					_starpu_enforce_deps_starting_from_task(j);
-				} else
+				}
+				else
 					STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
 
