@@ -1,9 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2015  Université de Bordeaux
+ * Copyright (C) 2010-2017  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012, 2013, 2015  Centre National de la Recherche Scientifique
- * Copyright (C) 2012 INRIA
+ * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2016, 2017  CNRS
+ * Copyright (C) 2012, 2016  Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,8 +22,6 @@
 #include <datawizard/interfaces/data_interface.h>
 #include <core/task.h>
 
-static void starpu_data_create_children(starpu_data_handle_t handle, unsigned nchildren, struct starpu_data_filter *f);
-
 /*
  * This function applies a data filter on all the elements of a partition
  */
@@ -68,6 +66,18 @@ void starpu_data_map_filters(starpu_data_handle_t root_handle, unsigned nfilters
 	va_end(pa);
 }
 
+void fstarpu_data_map_filters(starpu_data_handle_t root_handle, int nfilters, struct starpu_data_filter **filters)
+{
+	int i;
+	assert(nfilters >= 0);
+	for (i = 0; i < nfilters; i++)
+	{
+		struct starpu_data_filter *next_filter = filters[i];
+		STARPU_ASSERT(next_filter);
+		map_filter(root_handle, next_filter);
+	}
+}
+
 int starpu_data_get_nb_children(starpu_data_handle_t handle)
 {
         return handle->nchildren;
@@ -114,29 +124,58 @@ starpu_data_handle_t starpu_data_vget_sub_data(starpu_data_handle_t root_handle,
 	return current_handle;
 }
 
-void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_data_filter *f)
+starpu_data_handle_t fstarpu_data_get_sub_data(starpu_data_handle_t root_handle, int depth, int *indices)
+{
+	STARPU_ASSERT(root_handle);
+	starpu_data_handle_t current_handle = root_handle;
+
+	STARPU_ASSERT(depth >= 0);
+	/* the variable number of argument must correlate the depth in the tree */
+	int i;
+	for (i = 0; i < depth; i++)
+	{
+		int next_child;
+		next_child = indices[i];
+		STARPU_ASSERT(next_child >= 0);
+
+		STARPU_ASSERT_MSG(current_handle->nchildren != 0, "Data %p has to be partitioned before accessing children", current_handle);
+		STARPU_ASSERT_MSG((unsigned) next_child < current_handle->nchildren, "Bogus child number %d, data %p only has %u children", next_child, current_handle, current_handle->nchildren);
+
+		current_handle = &current_handle->children[next_child];
+	}
+
+	return current_handle;
+}
+
+static unsigned _starpu_data_partition_nparts(starpu_data_handle_t initial_handle, struct starpu_data_filter *f)
+{
+	/* how many parts ? */
+	if (f->get_nchildren)
+	  return f->get_nchildren(f, initial_handle);
+	else
+	  return f->nchildren;
+
+}
+
+static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_data_handle_t *childrenp, unsigned nparts, struct starpu_data_filter *f, int inherit_state)
 {
-	unsigned nparts;
 	unsigned i;
 	unsigned node;
 
 	/* first take care to properly lock the data header */
 	_starpu_spin_lock(&initial_handle->header_lock);
 
-	STARPU_ASSERT_MSG(initial_handle->nchildren == 0, "there should not be mutiple filters applied on the same data %p, futher filtering has to be done on children", initial_handle);
-
-	/* how many parts ? */
-	if (f->get_nchildren)
-	  nparts = f->get_nchildren(f, initial_handle);
-	else
-	  nparts = f->nchildren;
+	initial_handle->nplans++;
 
 	STARPU_ASSERT_MSG(nparts > 0, "Partitioning data %p in 0 piece does not make sense", initial_handle);
 
 	/* allocate the children */
-	starpu_data_create_children(initial_handle, nparts, f);
-
-	unsigned nworkers = starpu_worker_get_count();
+	if (inherit_state)
+	{
+		_STARPU_CALLOC(initial_handle->children, nparts, sizeof(struct _starpu_data_state));
+		/* this handle now has children */
+		initial_handle->nchildren = nparts;
+	}
 
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
@@ -147,18 +186,47 @@ void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_da
 	{
 		/* This is lazy allocation, allocate it now in main RAM, so as
 		 * to have somewhere to gather pieces later */
-		int ret = _starpu_allocate_memory_on_node(initial_handle, &initial_handle->per_node[0], 0);
+		/* FIXME: mark as unevictable! */
+		int ret = _starpu_allocate_memory_on_node(initial_handle, &initial_handle->per_node[STARPU_MAIN_RAM], 0);
+#ifdef STARPU_DEVEL
+#warning we should reclaim memory if allocation failed
+#endif
 		STARPU_ASSERT(!ret);
 	}
 
+	_starpu_data_unregister_ram_pointer(initial_handle);
+
+	if (nparts && !inherit_state)
+	{
+		STARPU_ASSERT_MSG(childrenp, "Passing NULL pointer for parameter childrenp while parameter inherit_state is 0");
+	}
+
 	for (i = 0; i < nparts; i++)
 	{
-		starpu_data_handle_t child =
-			starpu_data_get_child(initial_handle, i);
+		starpu_data_handle_t child;
 
+		if (inherit_state)
+			child = &initial_handle->children[i];
+		else
+			child = childrenp[i];
 		STARPU_ASSERT(child);
 
+		struct starpu_data_interface_ops *ops;
+
+		/* each child may have his own interface type */
+		/* what's this child's interface ? */
+		if (f->get_child_ops)
+			ops = f->get_child_ops(f, i);
+		else
+			ops = initial_handle->ops;
+
+		_starpu_data_handle_init(child, ops, initial_handle->mf_node);
+
 		child->nchildren = 0;
+		child->nplans = 0;
+		child->switch_cl = NULL;
+		child->partitioned = 0;
+		child->readonly = 0;
                 child->mpi_data = initial_handle->mpi_data;
 		child->root_handle = initial_handle->root_handle;
 		child->father_handle = initial_handle;
@@ -168,13 +236,14 @@ void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_da
 		child->is_not_important = initial_handle->is_not_important;
 		child->wt_mask = initial_handle->wt_mask;
 		child->home_node = initial_handle->home_node;
-		child->is_readonly = initial_handle->is_readonly;
 
 		/* initialize the chunk lock */
 		_starpu_data_requester_list_init(&child->req_list);
 		_starpu_data_requester_list_init(&child->reduction_req_list);
 		child->reduction_tmp_handles = NULL;
+		child->write_invalidation_req = NULL;
 		child->refcnt = 0;
+		child->unlocking_reqs = 0;
 		child->busy_count = 0;
 		child->busy_waiting = 0;
 		STARPU_PTHREAD_MUTEX_INIT(&child->busy_mutex, NULL);
@@ -183,6 +252,7 @@ void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_da
 		_starpu_spin_init(&child->header_lock);
 
 		child->sequential_consistency = initial_handle->sequential_consistency;
+		child->initialized = initial_handle->initialized;
 
 		STARPU_PTHREAD_MUTEX_INIT(&child->sequential_consistency_mutex, NULL);
 		child->last_submitted_mode = STARPU_R;
@@ -206,6 +276,13 @@ void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_da
 		child->last_submitted_ghost_accessors_id = NULL;
 #endif
 
+		if (_starpu_global_arbiter)
+			/* Just for testing purpose */
+			starpu_data_assign_arbiter(child, _starpu_global_arbiter);
+		else
+			child->arbiter = NULL;
+		_starpu_data_requester_list_init(&child->arbitered_req_list);
+
 		for (node = 0; node < STARPU_MAXNODES; node++)
 		{
 			struct _starpu_data_replicate *initial_replicate;
@@ -214,57 +291,45 @@ void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_da
 			initial_replicate = &initial_handle->per_node[node];
 			child_replicate = &child->per_node[node];
 
-			child_replicate->state = initial_replicate->state;
-			child_replicate->allocated = initial_replicate->allocated;
+			if (inherit_state)
+				child_replicate->state = initial_replicate->state;
+			else
+				child_replicate->state = STARPU_INVALID;
+			if (inherit_state || !initial_replicate->automatically_allocated)
+				child_replicate->allocated = initial_replicate->allocated;
+			else
+				child_replicate->allocated = 0;
 			/* Do not allow memory reclaiming within the child for parent bits */
 			child_replicate->automatically_allocated = 0;
 			child_replicate->refcnt = 0;
 			child_replicate->memory_node = node;
 			child_replicate->relaxed_coherency = 0;
-			child_replicate->initialized = initial_replicate->initialized;
+			if (inherit_state)
+				child_replicate->initialized = initial_replicate->initialized;
+			else
+				child_replicate->initialized = 0;
 
 			/* update the interface */
 			void *initial_interface = starpu_data_get_interface_on_node(initial_handle, node);
 			void *child_interface = starpu_data_get_interface_on_node(child, node);
 
+			STARPU_ASSERT_MSG(!(!inherit_state && child_replicate->automatically_allocated && child_replicate->allocated), "partition planning is currently not supported when handle has some automatically allocated buffers");
 			f->filter_func(initial_interface, child_interface, f, i, nparts);
 		}
 
-		unsigned worker;
-		for (worker = 0; worker < nworkers; worker++)
-		{
-			struct _starpu_data_replicate *child_replicate;
-			child_replicate = &child->per_worker[worker];
-
-			child_replicate->state = STARPU_INVALID;
-			child_replicate->allocated = 0;
-			child_replicate->automatically_allocated = 0;
-			child_replicate->refcnt = 0;
-			child_replicate->memory_node = starpu_worker_get_memory_node(worker);
-			child_replicate->requested = 0;
-
-			for (node = 0; node < STARPU_MAXNODES; node++)
-			{
-				child_replicate->request[node] = NULL;
-			}
-
-			child_replicate->relaxed_coherency = 1;
-			child_replicate->initialized = 0;
-
-			/* duplicate  the content of the interface on node 0 */
-			memcpy(child_replicate->data_interface, child->per_node[0].data_interface, child->ops->interface_size);
-		}
+		child->per_worker = NULL;
+		child->user_data = NULL;
 
 		/* We compute the size and the footprint of the child once and
 		 * store it in the handle */
 		child->footprint = _starpu_compute_data_footprint(child);
 
 		void *ptr;
-		ptr = starpu_data_handle_to_pointer(child, 0);
+		ptr = starpu_data_handle_to_pointer(child, STARPU_MAIN_RAM);
 		if (ptr != NULL)
-		{
 			_starpu_data_register_ram_pointer(child, ptr);
-		}
+
+		_STARPU_TRACE_HANDLE_DATA_REGISTER(child);
 	}
 	/* now let the header */
 	_starpu_spin_unlock(&initial_handle->header_lock);
@@ -284,6 +349,7 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 	unsigned nworkers = starpu_worker_get_count();
 	unsigned node;
 	unsigned sizes[root_handle->nchildren];
+	void *ptr;
 
 	_STARPU_TRACE_START_UNPARTITION(root_handle, gathering_node);
 	_starpu_spin_lock(&root_handle->header_lock);
@@ -309,11 +375,13 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 			struct starpu_codelet cl =
 			{
 				.where = STARPU_CPU,
-				.cpu_funcs = { _starpu_empty_codelet_function},
+				.cpu_funcs = { _starpu_empty_codelet_function },
 				.modes = { STARPU_RW },
 				.nbuffers = 1
 			};
 			struct starpu_task *task = starpu_task_create();
+			task->name = "convert_data";
+
 			STARPU_TASK_SET_HANDLE(task, child_handle, 0);
 			task->cl = &cl;
 			task->synchronous = 1;
@@ -357,6 +425,7 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 
 		_starpu_data_unregister_ram_pointer(child_handle);
 
+		if (child_handle->per_worker)
 		for (worker = 0; worker < nworkers; worker++)
 		{
 			struct _starpu_data_replicate *local = &child_handle->per_worker[worker];
@@ -368,6 +437,10 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 		_starpu_memory_stats_free(child_handle);
 	}
 
+	ptr = starpu_data_handle_to_pointer(root_handle, STARPU_MAIN_RAM);
+	if (ptr != NULL)
+		_starpu_data_register_ram_pointer(root_handle, ptr);
+
 	/* the gathering_node should now have a valid copy of all the children.
 	 * For all nodes, if the node had all copies and none was locally
 	 * allocated then the data is still valid there, else, it's invalidated
@@ -439,16 +512,24 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 		_starpu_data_free_interfaces(child_handle);
 		_starpu_spin_unlock(&child_handle->header_lock);
 		_starpu_spin_destroy(&child_handle->header_lock);
+	}
 
+	for (child = 0; child < root_handle->nchildren; child++)
+	{
+		starpu_data_handle_t child_handle = starpu_data_get_child(root_handle, child);
+		_starpu_data_clear_implicit(child_handle);
 		STARPU_PTHREAD_MUTEX_DESTROY(&child_handle->busy_mutex);
 		STARPU_PTHREAD_COND_DESTROY(&child_handle->busy_cond);
 		STARPU_PTHREAD_MUTEX_DESTROY(&child_handle->sequential_consistency_mutex);
+
+		_STARPU_TRACE_HANDLE_DATA_UNREGISTER(child_handle);
 	}
 
 	/* there is no child anymore */
 	starpu_data_handle_t children = root_handle->children;
 	root_handle->children = NULL;
 	root_handle->nchildren = 0;
+	root_handle->nplans--;
 
 	/* now the parent may be used again so we release the lock */
 	_starpu_spin_unlock(&root_handle->header_lock);
@@ -458,31 +539,194 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 	_STARPU_TRACE_END_UNPARTITION(root_handle, gathering_node);
 }
 
-/* each child may have his own interface type */
-static void starpu_data_create_children(starpu_data_handle_t handle, unsigned nchildren, struct starpu_data_filter *f)
+void starpu_data_partition(starpu_data_handle_t initial_handle, struct starpu_data_filter *f)
 {
-	handle->children = (struct _starpu_data_state *) calloc(nchildren, sizeof(struct _starpu_data_state));
-	STARPU_ASSERT(handle->children);
+	unsigned nparts = _starpu_data_partition_nparts(initial_handle, f);
+	STARPU_ASSERT_MSG(initial_handle->nchildren == 0, "there should not be mutiple filters applied on the same data %p, futher filtering has to be done on children", initial_handle);
+	STARPU_ASSERT_MSG(initial_handle->nplans == 0, "partition planning and synchronous partitioning is not supported");
 
-	unsigned child;
+	initial_handle->children = NULL;
 
-	for (child = 0; child < nchildren; child++)
+	/* Make sure to wait for previous tasks working on the whole data */
+	starpu_data_acquire_on_node(initial_handle, STARPU_ACQUIRE_NO_NODE, initial_handle->initialized?STARPU_RW:STARPU_W);
+	starpu_data_release_on_node(initial_handle, STARPU_ACQUIRE_NO_NODE);
+
+	_starpu_data_partition(initial_handle, NULL, nparts, f, 1);
+}
+
+void starpu_data_partition_plan(starpu_data_handle_t initial_handle, struct starpu_data_filter *f, starpu_data_handle_t *childrenp)
+{
+	unsigned i;
+	unsigned nparts = _starpu_data_partition_nparts(initial_handle, f);
+	STARPU_ASSERT_MSG(initial_handle->nchildren == 0, "partition planning and synchronous partitioning is not supported");
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	struct starpu_codelet *cl = initial_handle->switch_cl;
+	int home_node = initial_handle->home_node;
+	if (home_node == -1)
+		/* Nothing better for now */
+		/* TODO: pass -1, and make _starpu_fetch_nowhere_task_input
+		 * really call _starpu_fetch_data_on_node, and make that update
+		 * the coherency.
+		 */
+		home_node = STARPU_MAIN_RAM;
+
+	for (i = 0; i < nparts; i++)
+		_STARPU_CALLOC(childrenp[i], 1, sizeof(struct _starpu_data_state));
+	_starpu_data_partition(initial_handle, childrenp, nparts, f, 0);
+
+	if (!cl)
 	{
-		starpu_data_handle_t handle_child;
-		struct starpu_data_interface_ops *ops;
-
-		/* what's this child's interface ? */
-		if (f->get_child_ops)
-		  ops = f->get_child_ops(f, child);
-		else
-		  ops = handle->ops;
-
-		handle_child = &handle->children[child];
-		_starpu_data_handle_init(handle_child, ops, handle->mf_node);
+		/* Create a codelet that will make the coherency on the home node */
+		_STARPU_CALLOC(initial_handle->switch_cl, 1, sizeof(*initial_handle->switch_cl));
+		cl = initial_handle->switch_cl;
+		cl->where = STARPU_NOWHERE;
+		cl->nbuffers = STARPU_VARIABLE_NBUFFERS;
+		cl->name = "data_partition_switch";
+		cl->specific_nodes = 1;
 	}
+	if (initial_handle->switch_cl_nparts < nparts)
+	{
+		/* First initialization, or previous initialization was with fewer parts, enlarge it */
+		_STARPU_REALLOC(cl->dyn_nodes, (nparts+1) * sizeof(*cl->dyn_nodes));
+		for (i = initial_handle->switch_cl_nparts; i < nparts+1; i++)
+			cl->dyn_nodes[i] = home_node;
+		initial_handle->switch_cl_nparts = nparts;
+	}
+}
 
-	/* this handle now has children */
-	handle->nchildren = nchildren;
+void starpu_data_partition_clean(starpu_data_handle_t root_handle, unsigned nparts, starpu_data_handle_t *children)
+{
+	unsigned i;
+
+	for (i = 0; i < nparts; i++)
+		starpu_data_unregister_submit(children[i]);
+
+	_starpu_spin_lock(&root_handle->header_lock);
+	root_handle->nplans--;
+	_starpu_spin_unlock(&root_handle->header_lock);
+}
+
+void starpu_data_partition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
+{
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	_starpu_spin_lock(&initial_handle->header_lock);
+	STARPU_ASSERT_MSG(initial_handle->partitioned == 0, "One can't submit several partition plannings at the same time");
+	STARPU_ASSERT_MSG(initial_handle->readonly == 0, "One can't submit a partition planning while a readonly partitioning is active");
+	initial_handle->partitioned++;
+	_starpu_spin_unlock(&initial_handle->header_lock);
+
+	if (!initial_handle->initialized)
+		/* No need for coherency, it is not initialized */
+		return;
+	unsigned i;
+	struct starpu_data_descr descr[nparts];
+	for (i = 0; i < nparts; i++)
+	{
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		descr[i].handle = children[i];
+		descr[i].mode = STARPU_W;
+	}
+	/* TODO: assert nparts too */
+	starpu_task_insert(initial_handle->switch_cl, STARPU_RW, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts, 0);
+	starpu_data_invalidate_submit(initial_handle);
+}
+
+void starpu_data_partition_readonly_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
+{
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	_starpu_spin_lock(&initial_handle->header_lock);
+	STARPU_ASSERT_MSG(initial_handle->partitioned == 0 || initial_handle->readonly, "One can't submit a readonly partition planning at the same time as a readwrite partition planning");
+	initial_handle->partitioned++;
+	initial_handle->readonly = 1;
+	_starpu_spin_unlock(&initial_handle->header_lock);
+
+	STARPU_ASSERT_MSG(initial_handle->initialized, "It is odd to read-only-partition a data which does not have a value yet");
+	unsigned i;
+	struct starpu_data_descr descr[nparts];
+	for (i = 0; i < nparts; i++)
+	{
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		descr[i].handle = children[i];
+		descr[i].mode = STARPU_W;
+	}
+	/* TODO: assert nparts too */
+	starpu_task_insert(initial_handle->switch_cl, STARPU_R, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts, 0);
+}
+
+void starpu_data_partition_readwrite_upgrade_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
+{
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	_starpu_spin_lock(&initial_handle->header_lock);
+	STARPU_ASSERT_MSG(initial_handle->partitioned == 1, "One can't upgrade a readonly partition planning to readwrite while other readonly partition plannings are active");
+	STARPU_ASSERT_MSG(initial_handle->readonly == 1, "One can only upgrade a readonly partition planning");
+	initial_handle->readonly = 0;
+	_starpu_spin_unlock(&initial_handle->header_lock);
+
+	unsigned i;
+	struct starpu_data_descr descr[nparts];
+	for (i = 0; i < nparts; i++)
+	{
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		descr[i].handle = children[i];
+		descr[i].mode = STARPU_W;
+	}
+	/* TODO: assert nparts too */
+	starpu_task_insert(initial_handle->switch_cl, STARPU_RW, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts, 0);
+	starpu_data_invalidate_submit(initial_handle);
+}
+
+void starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node)
+{
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	STARPU_ASSERT_MSG(gather_node == initial_handle->home_node || gather_node == -1, "gathering node different from home node is currently not supported");
+	_starpu_spin_lock(&initial_handle->header_lock);
+	STARPU_ASSERT_MSG(initial_handle->partitioned >= 1, "No partition planning is active for this handle");
+	initial_handle->partitioned--;
+	if (!initial_handle->partitioned)
+		initial_handle->readonly = 0;
+	_starpu_spin_unlock(&initial_handle->header_lock);
+
+	unsigned i, n;
+	struct starpu_data_descr descr[nparts];
+	for (i = 0, n = 0; i < nparts; i++)
+	{
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		if (!children[i]->initialized)
+			/* Dropped value, do not care about coherency for this one */
+			continue;
+		descr[n].handle = children[i];
+		descr[n].mode = STARPU_RW;
+		n++;
+	}
+	/* TODO: assert nparts too */
+	starpu_task_insert(initial_handle->switch_cl, STARPU_W, initial_handle, STARPU_DATA_MODE_ARRAY, descr, n, 0);
+	for (i = 0; i < nparts; i++)
+		starpu_data_invalidate_submit(children[i]);
+}
+
+void starpu_data_unpartition_readonly_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node)
+{
+	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
+	STARPU_ASSERT_MSG(gather_node == initial_handle->home_node || gather_node == -1, "gathering node different from home node is currently not supported");
+	_starpu_spin_lock(&initial_handle->header_lock);
+	STARPU_ASSERT_MSG(initial_handle->partitioned >= 1, "No partition planning is active for this handle");
+	initial_handle->readonly = 1;
+	_starpu_spin_unlock(&initial_handle->header_lock);
+
+	unsigned i, n;
+	struct starpu_data_descr descr[nparts];
+	for (i = 0, n = 0; i < nparts; i++)
+	{
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		if (!children[i]->initialized)
+			/* Dropped value, do not care about coherency for this one */
+			continue;
+		descr[n].handle = children[i];
+		descr[n].mode = STARPU_R;
+		n++;
+	}
+	/* TODO: assert nparts too */
+	starpu_task_insert(initial_handle->switch_cl, STARPU_W, initial_handle, STARPU_DATA_MODE_ARRAY, descr, n, 0);
 }
 
 /*
