@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012  Université de Bordeaux 1
- * Copyright (C) 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012-2014  Université de Bordeaux
+ * Copyright (C) 2011, 2012, 2013, 2014, 2017  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,47 +25,25 @@
 #include <starpu_opencl.h>
 #include <drivers/opencl/driver_opencl.h>
 
-static int dummy_copy(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
-#ifdef STARPU_USE_CUDA
-static int dummy_cuda_copy_async(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, cudaStream_t stream);
-#endif
-#ifdef STARPU_USE_OPENCL
-static int dummy_opencl_copy_async(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *_event);
-#endif
+static int dummy_copy(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
 
-static struct starpu_data_copy_methods void_copy_data_methods_s =
+static const struct starpu_data_copy_methods void_copy_data_methods_s =
 {
-	.ram_to_ram = dummy_copy,
-	.ram_to_spu = dummy_copy,
-#ifdef STARPU_USE_CUDA
-	.ram_to_cuda = dummy_copy,
-	.cuda_to_ram = dummy_copy,
-	.cuda_to_cuda = dummy_copy,
-	.ram_to_cuda_async = dummy_cuda_copy_async,
-	.cuda_to_ram_async = dummy_cuda_copy_async,
-	.cuda_to_cuda_async = dummy_cuda_copy_async,
-#endif
-#ifdef STARPU_USE_OPENCL
-	.ram_to_opencl = dummy_copy,
-	.opencl_to_ram = dummy_copy,
-        .ram_to_opencl_async = dummy_opencl_copy_async,
-	.opencl_to_ram_async = dummy_opencl_copy_async,
-#endif
-	.cuda_to_spu = dummy_copy,
-	.spu_to_ram = dummy_copy,
-	.spu_to_cuda = dummy_copy,
-	.spu_to_spu = dummy_copy
+	.any_to_any = dummy_copy,
 };
 
-static void register_void_handle(starpu_data_handle_t handle, uint32_t home_node, void *data_interface);
-static ssize_t allocate_void_buffer_on_node(void *data_interface_, uint32_t dst_node);
-static void free_void_buffer_on_node(void *data_interface, uint32_t node);
+static void register_void_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static starpu_ssize_t allocate_void_buffer_on_node(void *data_interface_, unsigned dst_node);
+static void free_void_buffer_on_node(void *data_interface, unsigned node);
 static size_t void_interface_get_size(starpu_data_handle_t handle);
 static uint32_t footprint_void_interface_crc32(starpu_data_handle_t handle);
 static int void_compare(void *data_interface_a, void *data_interface_b);
 static void display_void_interface(starpu_data_handle_t handle, FILE *f);
+static int pack_void_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int unpack_void_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
+static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
-static struct starpu_data_interface_ops interface_void_ops =
+struct starpu_data_interface_ops starpu_interface_void_ops =
 {
 	.register_data_handle = register_void_handle,
 	.allocate_data_on_node = allocate_void_buffer_on_node,
@@ -76,11 +54,15 @@ static struct starpu_data_interface_ops interface_void_ops =
 	.compare = void_compare,
 	.interfaceid = STARPU_VOID_INTERFACE_ID,
 	.interface_size = 0,
-	.display = display_void_interface
+	.display = display_void_interface,
+	.pack_data = pack_void_handle,
+	.unpack_data = unpack_void_handle,
+	.describe = describe,
+	.name = "STARPU_VOID_INTERFACE"
 };
 
 static void register_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
-				uint32_t home_node STARPU_ATTRIBUTE_UNUSED,
+				unsigned home_node STARPU_ATTRIBUTE_UNUSED,
 				void *data_interface STARPU_ATTRIBUTE_UNUSED)
 {
 	/* Since there is no real data to register, we don't do anything */
@@ -89,7 +71,7 @@ static void register_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UN
 /* declare a new data with the void interface */
 void starpu_void_data_register(starpu_data_handle_t *handleptr)
 {
-	starpu_data_register(handleptr, 0, NULL, &interface_void_ops);
+	starpu_data_register(handleptr, STARPU_MAIN_RAM, NULL, &starpu_interface_void_ops);
 }
 
 
@@ -111,6 +93,24 @@ static void display_void_interface(starpu_data_handle_t handle STARPU_ATTRIBUTE_
 	fprintf(f, "void\t");
 }
 
+static int pack_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
+			    unsigned node STARPU_ATTRIBUTE_UNUSED,
+			    void **ptr,
+			    starpu_ssize_t *count)
+{
+	*count = 0;
+	*ptr = NULL;
+	return 0;
+}
+
+static int unpack_void_handle(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED,
+			      unsigned node STARPU_ATTRIBUTE_UNUSED,
+			      void *ptr STARPU_ATTRIBUTE_UNUSED,
+			      size_t count STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0;
+}
+
 static size_t void_interface_get_size(starpu_data_handle_t handle STARPU_ATTRIBUTE_UNUSED)
 {
 	return 0;
@@ -119,15 +119,15 @@ static size_t void_interface_get_size(starpu_data_handle_t handle STARPU_ATTRIBU
 /* memory allocation/deallocation primitives for the void interface */
 
 /* returns the size of the allocated area */
-static ssize_t allocate_void_buffer_on_node(void *data_interface STARPU_ATTRIBUTE_UNUSED,
-					uint32_t dst_node STARPU_ATTRIBUTE_UNUSED)
+static starpu_ssize_t allocate_void_buffer_on_node(void *data_interface STARPU_ATTRIBUTE_UNUSED,
+						   unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
 {
 	/* Successfuly allocated 0 bytes */
 	return 0;
 }
 
 static void free_void_buffer_on_node(void *data_interface STARPU_ATTRIBUTE_UNUSED ,
-					uint32_t node STARPU_ATTRIBUTE_UNUSED)
+					unsigned node STARPU_ATTRIBUTE_UNUSED)
 {
 	/* There is no buffer actually */
 }
@@ -135,29 +135,13 @@ static void free_void_buffer_on_node(void *data_interface STARPU_ATTRIBUTE_UNUSE
 static int dummy_copy(void *src_interface STARPU_ATTRIBUTE_UNUSED,
 			unsigned src_node STARPU_ATTRIBUTE_UNUSED,
 			void *dst_interface STARPU_ATTRIBUTE_UNUSED,
-			unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
+			unsigned dst_node STARPU_ATTRIBUTE_UNUSED,
+			void *async_data STARPU_ATTRIBUTE_UNUSED)
 {
 	return 0;
 }
 
-#ifdef STARPU_USE_CUDA
-static int dummy_cuda_copy_async(void *src_interface STARPU_ATTRIBUTE_UNUSED,
-				unsigned src_node STARPU_ATTRIBUTE_UNUSED,
-				void *dst_interface STARPU_ATTRIBUTE_UNUSED,
-				unsigned dst_node STARPU_ATTRIBUTE_UNUSED,
-				cudaStream_t stream __attribute__ ((unused)))
+static starpu_ssize_t describe(void *data_interface STARPU_ATTRIBUTE_UNUSED, char *buf, size_t size)
 {
-	return 0;
+	return snprintf(buf, size, "0");
 }
-#endif // STARPU_USE_CUDA
-
-#ifdef STARPU_USE_OPENCL
-static int dummy_opencl_copy_async(void *src_interface STARPU_ATTRIBUTE_UNUSED,
-					unsigned src_node STARPU_ATTRIBUTE_UNUSED,
-					void *dst_interface STARPU_ATTRIBUTE_UNUSED,
-					unsigned dst_node STARPU_ATTRIBUTE_UNUSED,
-					void *_event STARPU_ATTRIBUTE_UNUSED)
-{
-	return 0;
-}
-#endif // STARPU_USE_OPENCL
