@@ -1,9 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2014  Université de Bordeaux
+ * Copyright (C) 2010-2017  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010-2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2014  CNRS
  * Copyright (C) 2011  Télécom-SudParis
+ * Copyright (C) 2014  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -30,6 +31,7 @@
 #include <core/sched_policy.h>
 #include <datawizard/memory_manager.h>
 #include <datawizard/malloc.h>
+#include <core/simgrid.h>
 
 #ifdef STARPU_HAVE_HWLOC
 #include <hwloc.h>
@@ -42,91 +44,31 @@
 #include <windows.h>
 #endif
 
-#ifdef STARPU_SIMGRID
-#include <core/simgrid.h>
-#endif
-
-#ifdef STARPU_SIMGRID
-void
-_starpu_cpu_discover_devices(struct _starpu_machine_config *config)
-{
-	config->topology.nhwcpus = _starpu_simgrid_get_nbhosts("CPU");
-}
-#elif defined(STARPU_HAVE_HWLOC)
-void
-_starpu_cpu_discover_devices(struct _starpu_machine_config *config)
-{
-	/* Discover the CPUs relying on the hwloc interface and fills CONFIG
-	 * accordingly. */
-
-	struct _starpu_machine_topology *topology = &config->topology;
-
-	config->cpu_depth = hwloc_get_type_depth (topology->hwtopology,
-						  HWLOC_OBJ_CORE);
-
-	/* Would be very odd */
-	STARPU_ASSERT(config->cpu_depth != HWLOC_TYPE_DEPTH_MULTIPLE);
-
-	if (config->cpu_depth == HWLOC_TYPE_DEPTH_UNKNOWN) {
-		/* unknown, using logical procesors as fallback */
-		_STARPU_DISP("Warning: The OS did not report CPU cores. Assuming there is only one hardware thread per core.\n");
-		config->cpu_depth = hwloc_get_type_depth(topology->hwtopology,
-							 HWLOC_OBJ_PU);
-	}
-
-	topology->nhwcpus = hwloc_get_nbobjs_by_depth (topology->hwtopology,
-						       config->cpu_depth);
-}
-
-#elif defined(HAVE_SYSCONF)
-void
-_starpu_cpu_discover_devices(struct _starpu_machine_config *config)
-{
-	/* Discover the CPUs relying on the sysconf(3) function and fills
-	 * CONFIG accordingly. */
-
-	config->topology.nhwcpus = sysconf(_SC_NPROCESSORS_ONLN);
-}
-
-#elif defined(__MINGW32__) || defined(__CYGWIN__)
-void
-_starpu_cpu_discover_devices(struct _starpu_machine_config *config)
-{
-	/* Discover the CPUs on Cygwin and MinGW systems. */
-
-	SYSTEM_INFO sysinfo;
-	GetSystemInfo(&sysinfo);
-	config->topology.nhwcpus = sysinfo.dwNumberOfProcessors;
-}
-#else
-#warning no way to know number of cores, assuming 1
-void
-_starpu_cpu_discover_devices(struct _starpu_machine_config *config)
-{
-	config->topology.nhwcpus = 1;
-}
-#endif
-
 
 /* Actually launch the job on a cpu worker.
  * Handle binding CPUs on cores.
  * In the case of a combined worker WORKER_TASK != J->TASK */
 
-static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_task, struct _starpu_worker *cpu_args, int rank, enum starpu_perfmodel_archtype perf_arch)
+static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_task, struct _starpu_worker *cpu_args, int rank, struct starpu_perfmodel_arch* perf_arch)
 {
 	int ret;
 	int is_parallel_task = (j->task_size > 1);
 	int profiling = starpu_profiling_status_get();
-	struct timespec codelet_start, codelet_end;
-
 	struct starpu_task *task = j->task;
 	struct starpu_codelet *cl = task->cl;
+#ifdef STARPU_OPENMP
+	/* At this point, j->continuation as been cleared as the task is being
+	 * woken up, thus we use j->discontinuous instead for the check */
+	const unsigned continuation_wake_up = j->discontinuous;
+#else
+	const unsigned continuation_wake_up = 0;
+#endif
 
 	STARPU_ASSERT(cl);
 
-	if (rank == 0)
+	if (rank == 0 && !continuation_wake_up)
 	{
-		ret = _starpu_fetch_task_input(j, 0);
+		ret = _starpu_fetch_task_input(j);
 		if (ret != 0)
 		{
 			/* there was not enough memory so the codelet cannot be executed right now ... */
@@ -145,7 +87,7 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 	}
 
 	/* Give profiling variable */
-	_starpu_driver_start_job(cpu_args, j, &codelet_start, rank, profiling);
+	_starpu_driver_start_job(cpu_args, j, perf_arch, rank, profiling);
 
 	/* In case this is a Fork-join parallel task, the worker does not
 	 * execute the kernel at all. */
@@ -158,22 +100,36 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 		STARPU_ASSERT_MSG(func, "when STARPU_CPU is defined in 'where', cpu_func or cpu_funcs has to be defined");
 		if (_starpu_get_disable_kernels() <= 0)
 		{
+			_STARPU_TRACE_START_EXECUTING();
 #ifdef STARPU_SIMGRID
-			_starpu_simgrid_execute_job(j, perf_arch, NAN);
+			if (cl->flags & STARPU_CODELET_SIMGRID_EXECUTE)
+				func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
+			else
+				_starpu_simgrid_submit_job(cpu_args->workerid, j, perf_arch, NAN, NULL, NULL, NULL);
 #else
 			func(_STARPU_TASK_GET_INTERFACES(task), task->cl_arg);
 #endif
+			_STARPU_TRACE_END_EXECUTING();
 		}
 		if (is_parallel_task && cl->type == STARPU_FORKJOIN)
 			/* rebind to single CPU */
-			_starpu_bind_thread_on_cpu(cpu_args->config, cpu_args->bindid);
+			_starpu_bind_thread_on_cpu(cpu_args->config, cpu_args->bindid, cpu_args->workerid);
 	}
 
-	_starpu_driver_end_job(cpu_args, j, perf_arch, &codelet_end, rank, profiling);
+	_starpu_driver_end_job(cpu_args, j, perf_arch, rank, profiling);
 
 	if (is_parallel_task)
 	{
 		STARPU_PTHREAD_BARRIER_WAIT(&j->after_work_barrier);
+#ifdef STARPU_SIMGRID
+		if (rank == 0)
+		{
+			/* Wait for other threads to exit barrier_wait so we
+			 * can safely drop the job structure */
+			MSG_process_sleep(0.0000001);
+			j->after_work_busy_barrier = 0;
+		}
+#else
 		ANNOTATE_HAPPENS_BEFORE(&j->after_work_busy_barrier);
 		(void) STARPU_ATOMIC_ADD(&j->after_work_busy_barrier, -1);
 		if (rank == 0)
@@ -188,19 +144,24 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 			}
 			ANNOTATE_HAPPENS_AFTER(&j->after_work_busy_barrier);
 		}
+#endif
 	}
 
 	if (rank == 0)
 	{
-		_starpu_driver_update_job_feedback(j, cpu_args,
-				perf_arch, &codelet_start, &codelet_end, profiling);
-		_starpu_push_task_output(j, 0);
+		_starpu_driver_update_job_feedback(j, cpu_args, perf_arch, profiling);
+#ifdef STARPU_OPENMP
+		if (!j->continuation)
+#endif
+		{
+			_starpu_push_task_output(j);
+		}
 	}
 
 	return 0;
 }
 
-static size_t _starpu_cpu_get_global_mem_size(int nodeid STARPU_ATTRIBUTE_UNUSED, struct _starpu_machine_config *config)
+static size_t _starpu_cpu_get_global_mem_size(int nodeid STARPU_ATTRIBUTE_UNUSED, struct _starpu_machine_config *config STARPU_ATTRIBUTE_UNUSED)
 {
 	size_t global_mem;
 	starpu_ssize_t limit;
@@ -247,19 +208,19 @@ int _starpu_cpu_driver_init(struct _starpu_worker *cpu_worker)
 {
 	int devid = cpu_worker->devid;
 
-	_starpu_worker_start(cpu_worker, _STARPU_FUT_CPU_KEY);
+	_starpu_driver_start(cpu_worker, _STARPU_FUT_CPU_KEY, 1);
 	/* FIXME: when we have NUMA support, properly turn node number into NUMA node number */
 	_starpu_memory_manager_set_global_memory_size(cpu_worker->memory_node, _starpu_cpu_get_global_mem_size(cpu_worker->memory_node, cpu_worker->config));
 
 	snprintf(cpu_worker->name, sizeof(cpu_worker->name), "CPU %d", devid);
 	snprintf(cpu_worker->short_name, sizeof(cpu_worker->short_name), "CPU %d", devid);
+	starpu_pthread_setname(cpu_worker->short_name);
 
-	cpu_worker->status = STATUS_UNKNOWN;
-
-	_STARPU_TRACE_WORKER_INIT_END;
+	_STARPU_TRACE_WORKER_INIT_END(cpu_worker->workerid);
 
 	/* tell the main thread that we are ready */
 	STARPU_PTHREAD_MUTEX_LOCK(&cpu_worker->mutex);
+	cpu_worker->status = STATUS_UNKNOWN;
 	cpu_worker->worker_is_initialized = 1;
 	STARPU_PTHREAD_COND_SIGNAL(&cpu_worker->ready_cond);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&cpu_worker->mutex);
@@ -271,9 +232,9 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 	unsigned memnode = cpu_worker->memory_node;
 	int workerid = cpu_worker->workerid;
 
-	_STARPU_TRACE_START_PROGRESS(memnode);
 	_starpu_datawizard_progress(memnode, 1);
-	_STARPU_TRACE_END_PROGRESS(memnode);
+	if (memnode != STARPU_MAIN_RAM)
+		_starpu_datawizard_progress(STARPU_MAIN_RAM, 1);
 
 	struct _starpu_job *j;
 	struct starpu_task *task;
@@ -297,8 +258,9 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 	int rank = 0;
 	int is_parallel_task = (j->task_size > 1);
 
-	enum starpu_perfmodel_archtype perf_arch;
+	struct starpu_perfmodel_arch* perf_arch;
 
+	_STARPU_TRACE_END_PROGRESS(memnode);
 	/* Get the rank in case it is a parallel task */
 	if (is_parallel_task)
 	{
@@ -306,20 +268,30 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 		rank = j->active_task_alias_count++;
 		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
-		struct _starpu_combined_worker *combined_worker;
-		combined_worker = _starpu_get_combined_worker_struct(j->combined_workerid);
+		if(j->combined_workerid != -1)
+		{
+			struct _starpu_combined_worker *combined_worker;
+			combined_worker = _starpu_get_combined_worker_struct(j->combined_workerid);
+			
+			cpu_worker->combined_workerid = j->combined_workerid;
+			cpu_worker->worker_size = combined_worker->worker_size;
+			cpu_worker->current_rank = rank;
+			perf_arch = &combined_worker->perf_arch;
+		}
+		else
+		{
+			struct _starpu_sched_ctx *sched_ctx = _starpu_sched_ctx_get_sched_ctx_for_worker_and_job(cpu_worker, j);
+			STARPU_ASSERT_MSG(sched_ctx != NULL, "there should be a worker %d in the ctx of this job \n", cpu_worker->workerid);
 
-		cpu_worker->combined_workerid = j->combined_workerid;
-		cpu_worker->worker_size = combined_worker->worker_size;
-		cpu_worker->current_rank = rank;
-		perf_arch = combined_worker->perf_arch;
+			perf_arch = &sched_ctx->perf_arch;
+		}
 	}
 	else
 	{
 		cpu_worker->combined_workerid = cpu_worker->workerid;
 		cpu_worker->worker_size = 1;
 		cpu_worker->current_rank = 0;
-		perf_arch = cpu_worker->perf_arch;
+		perf_arch = &cpu_worker->perf_arch;
 	}
 
 	_starpu_set_current_task(j->task);
@@ -336,6 +308,7 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 		{
 		case -EAGAIN:
 			_starpu_push_task_to_workers(task);
+			_STARPU_TRACE_START_PROGRESS(memnode);
 			return 0;
 		default:
 			STARPU_ABORT();
@@ -353,6 +326,7 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 
 	if (rank == 0)
 		_starpu_handle_job_termination(j);
+	_STARPU_TRACE_START_PROGRESS(memnode);
 	return 0;
 }
 
@@ -380,11 +354,13 @@ _starpu_cpu_worker(void *arg)
 	struct _starpu_worker *args = arg;
 
 	_starpu_cpu_driver_init(args);
+	_STARPU_TRACE_START_PROGRESS(args->memory_node);
 	while (_starpu_machine_is_running())
 	{
 		_starpu_may_pause();
 		_starpu_cpu_driver_run_once(args);
 	}
+	_STARPU_TRACE_END_PROGRESS(args->memory_node);
 	_starpu_cpu_driver_deinit(args);
 
 	return NULL;
