@@ -25,5 +25,6 @@
 #define _STARPU_CPU_ALPHA	1.0f
 #define _STARPU_CUDA_ALPHA	13.33f
 #define _STARPU_OPENCL_ALPHA	12.22f
-
+#define _STARPU_MIC_ALPHA	0.5f
+#define _STARPU_SCC_ALPHA	1.0f
 #endif /* _STARPU_PARAMETERS_H */
