@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2014, 2016-2017  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,11 +17,12 @@
 
 #include <config.h>
 #include <starpu.h>
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
 #include "../helper.h"
 
+/*
+ * Call acquire/release in competition with inserting task working on the same data
+ */
+
 #ifdef STARPU_QUICK_CHECK
 static unsigned ntasks = 10;
 #else
@@ -29,16 +30,14 @@ static unsigned ntasks = 10000;
 #endif
 
 #ifdef STARPU_USE_CUDA
-extern void increment_cuda(void *descr[], __attribute__ ((unused)) void *_args);
+extern void increment_cuda(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
 #endif
 #ifdef STARPU_USE_OPENCL
 extern void increment_opencl(void *buffers[], void *args);
 #endif
 
-void increment_cpu(void *descr[], __attribute__ ((unused)) void *_args)
+void increment_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	(*tokenptr)++;
 }
@@ -46,20 +45,24 @@ void increment_cpu(void *descr[], __attribute__ ((unused)) void *_args)
 static struct starpu_codelet increment_cl =
 {
 	.modes = { STARPU_RW },
-	.cpu_funcs = {increment_cpu, NULL},
+	.cpu_funcs = {increment_cpu},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {increment_cuda, NULL},
+	.cuda_funcs = {increment_cuda},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {increment_opencl, NULL},
+	.opencl_funcs = {increment_opencl},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
+	.cpu_funcs_name = {"increment_cpu"},
 	.nbuffers = 1
 };
 
 unsigned token = 0;
 starpu_data_handle_t token_handle;
 
-int increment_token()
+static
+int increment_token(void)
 {
 	int ret;
 	struct starpu_task *task = starpu_task_create();
@@ -70,9 +73,11 @@ int increment_token()
 	return ret;
 }
 
-void callback(void *arg __attribute__ ((unused)))
+static
+void callback(void *arg STARPU_ATTRIBUTE_UNUSED)
 {
-        starpu_data_release(token_handle);
+	token++;
+	starpu_data_release(token_handle);
 }
 
 #ifdef STARPU_USE_OPENCL
@@ -80,10 +85,10 @@ struct starpu_opencl_program opencl_program;
 #endif
 int main(int argc, char **argv)
 {
-	int i;
+	unsigned i;
 	int ret;
 
-        ret = starpu_init(NULL);
+        ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -92,14 +97,14 @@ int main(int argc, char **argv)
 						  &opencl_program, NULL);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_load_opencl_from_file");
 #endif
-	starpu_variable_data_register(&token_handle, 0, (uintptr_t)&token, sizeof(unsigned));
+	starpu_variable_data_register(&token_handle, STARPU_MAIN_RAM, (uintptr_t)&token, sizeof(unsigned));
 
         FPRINTF(stderr, "Token: %u\n", token);
 
 	for(i=0; i<ntasks; i++)
 	{
 		/* synchronize data in RAM */
-                ret = starpu_data_acquire(token_handle, STARPU_R);
+                ret = starpu_data_acquire(token_handle, STARPU_RW);
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_data_acquire");
 
                 token ++;
@@ -122,11 +127,11 @@ int main(int argc, char **argv)
 	starpu_shutdown();
 
         FPRINTF(stderr, "Token: %u\n", token);
-	if (token == ntasks * 2)
+	if (token == ntasks * 3)
 		ret = EXIT_SUCCESS;
 	else
 		ret = EXIT_FAILURE;
-	STARPU_RETURN(ret);
+	return ret;
 
 enodev:
 	starpu_data_unregister(token_handle);
