@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,12 +19,9 @@
 #include <starpu.h>
 #include "../helper.h"
 
-#ifdef STARPU_USE_CUDA
-#include <starpu_cuda.h>
-#endif
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
+/*
+ * Check that STARPU_REDUX works with a mere incrementation
+ */
 
 static unsigned var = 0;
 static starpu_data_handle_t handle;
@@ -44,14 +41,13 @@ static void redux_cuda_kernel(void *descr[], void *arg)
 	unsigned host_dst, host_src;
 
 	/* This is a dummy technique of course */
-	cudaMemcpy(&host_src, src, sizeof(unsigned), cudaMemcpyDeviceToHost);
-	cudaMemcpy(&host_dst, dst, sizeof(unsigned), cudaMemcpyDeviceToHost);
-	cudaThreadSynchronize();
+	cudaMemcpyAsync(&host_src, src, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	cudaMemcpyAsync(&host_dst, dst, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 
 	host_dst += host_src;
 
-	cudaMemcpy(dst, &host_dst, sizeof(unsigned), cudaMemcpyHostToDevice);
-	cudaThreadSynchronize();
+	cudaMemcpyAsync(dst, &host_dst, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
 }
 
 static void neutral_cuda_kernel(void *descr[], void *arg)
@@ -62,8 +58,7 @@ static void neutral_cuda_kernel(void *descr[], void *arg)
 
 	/* This is a dummy technique of course */
 	unsigned host_dst = 0;
-	cudaMemcpy(dst, &host_dst, sizeof(unsigned), cudaMemcpyHostToDevice);
-	cudaThreadSynchronize();
+	cudaMemcpyAsync(dst, &host_dst, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
 }
 #endif
 
@@ -87,7 +82,6 @@ static void redux_opencl_kernel(void *descr[], void *arg)
 	h_dst += h_src;
 
 	clEnqueueWriteBuffer(queue, d_dst, CL_TRUE, 0, sizeof(unsigned), (void *)&h_dst, 0, NULL, NULL);
-	clFinish(queue);
 }
 
 static void neutral_opencl_kernel(void *descr[], void *arg)
@@ -101,13 +95,10 @@ static void neutral_opencl_kernel(void *descr[], void *arg)
 	starpu_opencl_get_current_queue(&queue);
 
 	clEnqueueWriteBuffer(queue, d_dst, CL_TRUE, 0, sizeof(unsigned), (void *)&h_dst, 0, NULL, NULL);
-	clFinish(queue);
 }
 #endif
 
-
-
-static void redux_cpu_kernel(void *descr[], void *arg)
+void redux_cpu_kernel(void *descr[], void *arg)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -116,7 +107,7 @@ static void redux_cpu_kernel(void *descr[], void *arg)
 	*dst = *dst + *src;
 }
 
-static void neutral_cpu_kernel(void *descr[], void *arg)
+void neutral_cpu_kernel(void *descr[], void *arg)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -126,27 +117,33 @@ static void neutral_cpu_kernel(void *descr[], void *arg)
 
 static struct starpu_codelet redux_cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {redux_cuda_kernel, NULL},
+	.cuda_funcs = {redux_cuda_kernel},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {redux_opencl_kernel, NULL},
+	.opencl_funcs = {redux_opencl_kernel},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
-	.cpu_funcs = {redux_cpu_kernel, NULL},
+	.cpu_funcs = {redux_cpu_kernel},
+	.cpu_funcs_name = {"redux_cpu_kernel"},
+	.modes = {STARPU_RW, STARPU_R},
 	.nbuffers = 2
 };
 
 static struct starpu_codelet neutral_cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA,
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {neutral_cuda_kernel, NULL},
+	.cuda_funcs = {neutral_cuda_kernel},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {neutral_opencl_kernel, NULL},
+	.opencl_funcs = {neutral_opencl_kernel},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
-	.cpu_funcs = {neutral_cpu_kernel, NULL},
+	.cpu_funcs = {neutral_cpu_kernel},
+	.cpu_funcs_name = {"neutral_cpu_kernel"},
+	.modes = {STARPU_W},
 	.nbuffers = 1
 };
 
@@ -156,7 +153,7 @@ static struct starpu_codelet neutral_cl =
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void increment_opencl_kernel(void *descr[], void *cl_arg __attribute__((unused)))
+static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -169,7 +166,6 @@ static void increment_opencl_kernel(void *descr[], void *cl_arg __attribute__((u
 	clEnqueueReadBuffer(queue, d_token, CL_TRUE, 0, sizeof(unsigned), (void *)&h_token, 0, NULL, NULL);
 	h_token++;
 	clEnqueueWriteBuffer(queue, d_token, CL_TRUE, 0, sizeof(unsigned), (void *)&h_token, 0, NULL, NULL);
-	clFinish(queue);
 }
 #endif
 
@@ -183,17 +179,16 @@ static void increment_cuda_kernel(void *descr[], void *arg)
 	unsigned host_token;
 
 	/* This is a dummy technique of course */
-	cudaMemcpy(&host_token, tokenptr, sizeof(unsigned), cudaMemcpyDeviceToHost);
-	cudaThreadSynchronize();
+	cudaMemcpyAsync(&host_token, tokenptr, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 
 	host_token++;
 
-	cudaMemcpy(tokenptr, &host_token, sizeof(unsigned), cudaMemcpyHostToDevice);
-	cudaThreadSynchronize();
+	cudaMemcpyAsync(tokenptr, &host_token, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
 }
 #endif
 
-static void increment_cpu_kernel(void *descr[], void *arg)
+void increment_cpu_kernel(void *descr[], void *arg)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -203,14 +198,16 @@ static void increment_cpu_kernel(void *descr[], void *arg)
 
 static struct starpu_codelet increment_cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {increment_cuda_kernel, NULL},
+	.cuda_funcs = {increment_cuda_kernel},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {increment_opencl_kernel, NULL},
+	.opencl_funcs = {increment_opencl_kernel},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
-	.cpu_funcs = {increment_cpu_kernel, NULL},
+	.cpu_funcs = {increment_cpu_kernel},
+	.cpu_funcs_name = {"increment_cpu_kernel"},
 	.nbuffers = 1,
 	.modes = {STARPU_REDUX}
 };
@@ -219,11 +216,15 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	/* Not supported yet */
+	if (starpu_get_env_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
+		return STARPU_TEST_SKIPPED;
+
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	starpu_variable_data_register(&handle, 0, (uintptr_t)&var, sizeof(unsigned));
+	starpu_variable_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)&var, sizeof(unsigned));
 
 	starpu_data_set_reduction_methods(handle, &redux_cl, &neutral_cl);
 
@@ -247,7 +248,7 @@ int main(int argc, char **argv)
 			task->cl = &increment_cl;
 			task->handles[0] = handle;
 
-			int ret = starpu_task_submit(task);
+			ret = starpu_task_submit(task);
 			if (ret == -ENODEV) goto enodev;
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 		}
