@@ -22,7 +22,7 @@
  * A single handle can be given twice to a given kernel. In this case, it
  * should only be converted once.
  */
-#if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL)
+#if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL) || defined(STARPU_USE_MIC)
 extern struct stats global_stats;
 static int vector[NX]; static starpu_data_handle_t handle;
 
@@ -30,10 +30,13 @@ static struct starpu_codelet cl =
 {
 	.modes = { STARPU_RW, STARPU_RW },
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = { cuda_func},
+	.cuda_funcs = { cuda_func },
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = { opencl_func},
+	.opencl_funcs = { opencl_func },
+#endif
+#ifdef STARPU_USE_MIC
+	.mic_funcs = {mic_func},
 #endif
 	.nbuffers = 2,
 };
@@ -44,7 +47,7 @@ register_handle(void)
 	int i;
 	for (i = 0; i < NX; i++)
 		vector[i] = i;
-	starpu_multiformat_data_register(&handle, 0, vector, NX, &ops);
+	starpu_multiformat_data_register(&handle, STARPU_MAIN_RAM, vector, NX, &ops);
 }
 
 static void
@@ -66,6 +69,9 @@ create_and_submit_tasks(void)
 #ifdef STARPU_USE_OPENCL
 	cl.where |= STARPU_OPENCL;
 #endif
+#ifdef STARPU_USE_MIC
+	cl.where |= STARPU_MIC;
+#endif
 
 	task = starpu_task_create();
 	task->cl = &cl;
@@ -74,18 +80,22 @@ create_and_submit_tasks(void)
 
 	ret = starpu_task_submit(task);
 	if (ret == -ENODEV)
+	{
+		task->destroy = 0;
+		starpu_task_destroy(task);
 		return -ENODEV;
+	}
 
 	return 0;
 }
 #endif
 
 int
-main(void)
+main(int argc, char **argv)
 {
-#if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL)
+#if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL) || defined(STARPU_USE_MIC)
 	int err;
-	err = starpu_init(NULL);
+	err = starpu_initialize(NULL, &argc, &argv);
 	if (err == -ENODEV)
 		goto enodev;
 
@@ -120,6 +130,17 @@ main(void)
 
 	}
 	else
+#endif
+#if defined(STARPU_USE_MIC)
+	if (global_stats.mic == 1)
+	{
+		if (global_stats.cpu_to_mic == 1 &&
+			global_stats.mic_to_cpu == 1)
+			return EXIT_SUCCESS;
+		else
+			return EXIT_FAILURE;
+	}
+	else
 #endif
 	{
 		/* We should not get here */
