@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,6 +26,7 @@ extern void test_multiformat_cuda_func(void *buffers[], void *args);
 #ifdef STARPU_USE_OPENCL
 extern void test_multiformat_opencl_func(void *buffers[], void *args);
 #endif
+void test_multiformat_mic_func(void *buffers[], void *args);
 
 static struct point array_of_structs[N_ELEMENTS];
 static struct point array_of_structs_dummy[N_ELEMENTS];
@@ -42,9 +43,10 @@ struct test_config multiformat_config =
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_multiformat_opencl_func,
 #endif
+	.cpu_func_name = "test_multiformat_mic_func",
 	.handle        = &multiformat_handle,
 	.dummy_handle  = &multiformat_dummy_handle,
-	.copy_failed   = 0,
+	.copy_failed   = SUCCESS,
 	.name          = "multiformat_interface"
 };
 
@@ -54,7 +56,7 @@ test_multiformat_cpu_func(void *buffers[], void *args)
 	STARPU_SKIP_IF_VALGRIND;
 
 	struct point *aos;
-	unsigned int n, i;
+	int n, i;
 	int factor;
 
 	aos = (struct point *) STARPU_MULTIFORMAT_GET_CPU_PTR(buffers[0]);
@@ -63,10 +65,10 @@ test_multiformat_cpu_func(void *buffers[], void *args)
 
 	for (i = 0; i < n; i++)
 	{
-			FPRINTF(stderr, "(%d %d) [%d]", aos[i].x, aos[i].y, factor);
+		FPRINTF(stderr, "(%d %d) [%d]", aos[i].x, aos[i].y, factor);
 		if (aos[i].x != i * factor || aos[i].y != i * factor)
 		{
-			multiformat_config.copy_failed = 1;
+			multiformat_config.copy_failed = FAILURE;
 		}
 		aos[i].x = -aos[i].x;
 		aos[i].y = -aos[i].y;
@@ -74,7 +76,32 @@ test_multiformat_cpu_func(void *buffers[], void *args)
 	FPRINTF(stderr, "\n");
 }
 
+void test_multiformat_mic_func(void *buffers[], void *args)
+{
+	STARPU_SKIP_IF_VALGRIND;
 
+	printf("MIC\n");
+
+	struct struct_of_arrays *soa;
+	int n, i;
+	int factor;
+
+	soa = (struct struct_of_arrays *) STARPU_MULTIFORMAT_GET_MIC_PTR(buffers[0]);
+	n = STARPU_MULTIFORMAT_GET_NX(buffers[0]);
+	factor = *(int *) args;
+
+	for (i = 0; i < n; i++)
+	{
+		FPRINTF(stderr, "(%d %d) [%d]", soa->x[i], soa->y[i], factor);
+		if (soa->x[i] != i * factor || soa->y[i] != i * factor)
+		{
+			multiformat_config.copy_failed = 1;
+		}
+		soa->x[i] = -soa->x[i];
+		soa->y[i] = -soa->y[i];
+	}
+	FPRINTF(stderr, "\n");
+}
 
 #ifdef STARPU_USE_CUDA
 extern struct starpu_codelet cpu_to_cuda_cl;
@@ -86,6 +113,11 @@ extern struct starpu_codelet cpu_to_opencl_cl;
 extern struct starpu_codelet opencl_to_cpu_cl;
 #endif
 
+#ifdef STARPU_USE_MIC
+extern struct starpu_codelet cpu_to_mic_cl;
+extern struct starpu_codelet mic_to_cpu_cl;
+#endif
+
 struct starpu_multiformat_data_interface_ops format_ops =
 {
 #ifdef STARPU_USE_CUDA
@@ -97,6 +129,11 @@ struct starpu_multiformat_data_interface_ops format_ops =
 	.opencl_elemsize = 2 * sizeof(float),
 	.cpu_to_opencl_cl = &cpu_to_opencl_cl,
 	.opencl_to_cpu_cl = &opencl_to_cpu_cl,
+#endif
+#ifdef STARPU_USE_MIC
+	.mic_elemsize = 2 * sizeof(float),
+	.cpu_to_mic_cl = &cpu_to_mic_cl,
+	.mic_to_cpu_cl = &mic_to_cpu_cl,
 #endif
 	.cpu_elemsize = sizeof(struct point),
 };
@@ -112,12 +149,12 @@ register_data(void)
 		array_of_structs[i].y = i;
 	}
 	starpu_multiformat_data_register(&multiformat_handle,
-					 0,
+					 STARPU_MAIN_RAM,
 					 &array_of_structs,
 					 N_ELEMENTS,
 					 &format_ops);
 	starpu_multiformat_data_register(&multiformat_dummy_handle,
-					 0,
+					 STARPU_MAIN_RAM,
 					 &array_of_structs_dummy,
 					 N_ELEMENTS,
 					 &format_ops);
@@ -131,19 +168,18 @@ unregister_data(void)
 }
 
 int
-main(void)
+main(int argc, char **argv)
 {
 #ifdef STARPU_USE_CPU
 	int ret;
 	data_interface_test_summary *summary;
-	struct starpu_conf conf =
-	{
-		.ncpus   = -1,
-		.ncuda   = 2,
-		.nopencl = 1
-	};
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	conf.ncuda = 2;
+	conf.nopencl = 1;
+	conf.nmic = -1;
 
-	ret = starpu_init(&conf);
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV || starpu_cpu_worker_get_count() == 0)
 		return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
