@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011  INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,8 +23,8 @@ static starpu_data_handle_t variable_handle;
 static starpu_data_handle_t variable2_handle;
 
 /* Codelets */
-#ifdef STARPU_USE_CPU
-static void test_variable_cpu_func(void *buffers[], void *args);
+#if defined(STARPU_USE_CPU) || defined(STARPU_USE_MIC)
+void test_variable_cpu_func(void *buffers[], void *args);
 #endif
 #ifdef STARPU_USE_CUDA
 extern void test_variable_cuda_func(void *buffers[], void *args);
@@ -43,14 +43,17 @@ struct test_config variable_config =
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func  = test_variable_opencl_func,
+#endif
+#ifdef STARPU_USE_MIC
+	.cpu_func_name = "test_variable_cpu_func",
 #endif
 	.handle       = &variable_handle,
 	.dummy_handle = &variable2_handle,
-	.copy_failed  = 0,
+	.copy_failed  = SUCCESS,
 	.name         = "variable_interface"
 };
 
-static void
+void
 test_variable_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
@@ -62,7 +65,7 @@ test_variable_cpu_func(void *buffers[], void *args)
 	factor = *(int *)args;
 
 	if (*val != 42 * factor)
-		variable_config.copy_failed = 1;
+		variable_config.copy_failed = FAILURE;
 	else
 		*val *= -1;
 }
@@ -74,9 +77,9 @@ void register_data(void)
 	variable = 42;
 	variable2 = 12;
 
-	starpu_variable_data_register(&variable_handle, 0,
+	starpu_variable_data_register(&variable_handle, STARPU_MAIN_RAM,
 				      (uintptr_t) &variable, sizeof(variable));
-	starpu_variable_data_register(&variable2_handle, 0,
+	starpu_variable_data_register(&variable2_handle, STARPU_MAIN_RAM,
 				      (uintptr_t) &variable2, sizeof(variable2));
 }
 
@@ -88,19 +91,19 @@ void unregister_data(void)
 }
 
 int
-main(void)
+main(int argc, char **argv)
 {
 	int ret;
 	data_interface_test_summary *summary;
 
-	struct starpu_conf conf =
-	{
-		.ncpus = -1,
-		.ncuda = 2,
-		.nopencl = 1
-	};
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
 
-	ret = starpu_init(&conf);
+	conf.ncuda = 2;
+	conf.nopencl = 1;
+	conf.nmic = -1;
+
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV || starpu_cpu_worker_get_count() == 0)
 		return STARPU_TEST_SKIPPED;
 
