@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013  Université de Bordeaux
- * Copyright (C) 2012       inria
+ * Copyright (C) 2010-2012, 2014, 2016  Université de Bordeaux
+ * Copyright (C) 2012       INRIA
+ * Copyright (C) 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +24,11 @@
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Trigger lazy allocation by registering NULL, then setting a value, and
+ * then checking it
+ */
+
 #define VECTORSIZE	1024
 
 static starpu_data_handle_t v_handle;
@@ -40,7 +46,6 @@ static void cuda_memset_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_ar
 	unsigned length = STARPU_VECTOR_GET_NX(descr[0]);
 
 	cudaMemsetAsync(buf, 42, length, starpu_cuda_get_local_stream());
-	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 }
 #endif
 
@@ -50,7 +55,7 @@ static void opencl_memset_codelet(void *buffers[], void *args)
 	(void) args;
 
 	cl_command_queue queue;
-	int id = starpu_worker_get_id();
+	int id = starpu_worker_get_id_check();
 	int devid = starpu_worker_get_devid(id);
 	starpu_opencl_get_queue(devid, &queue);
 
@@ -69,11 +74,10 @@ static void opencl_memset_codelet(void *buffers[], void *args)
 			     0,      /* num_events_in_wait_list */
 			     NULL,   /* event_wait_list */
 			     NULL    /* event */);
-	clFinish(queue);
 }
 #endif
 
-static void cpu_memset_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_memset_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -88,10 +92,13 @@ static struct starpu_codelet memset_cl =
 	.cpu_funcs = {cpu_memset_codelet},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {cuda_memset_codelet},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {opencl_memset_codelet},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
+	.cpu_funcs_name = {"cpu_memset_codelet"},
 	.nbuffers = 1,
 	.modes = {STARPU_W}
 };
@@ -100,7 +107,7 @@ static struct starpu_codelet memset_cl =
  *	Check content
  */
 
-static void cpu_check_content_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_check_content_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -146,7 +153,7 @@ static void opencl_check_content_codelet(void *buffers[], void *args)
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_command_queue queue;
-	int id = starpu_worker_get_id();
+	int id = starpu_worker_get_id_check();
 	int devid = starpu_worker_get_devid(id);
 	starpu_opencl_get_queue(devid, &queue);
 
@@ -185,6 +192,7 @@ static struct starpu_codelet check_content_cl =
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {opencl_check_content_codelet},
 #endif
+	.cpu_funcs_name = {"cpu_check_content_codelet"},
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
@@ -194,22 +202,22 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	starpu_vector_data_register(&v_handle, (uint32_t)-1, (uintptr_t)NULL, VECTORSIZE, sizeof(char));
 
-	ret = starpu_insert_task(&memset_cl, STARPU_W, v_handle, 0);
+	ret = starpu_task_insert(&memset_cl, STARPU_W, v_handle, 0);
 	if (ret == -ENODEV)
 		return STARPU_TEST_SKIPPED;
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_insert_task");
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 
         ret = starpu_task_wait_for_all();
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait_for_all");
 
-	ret = starpu_insert_task(&check_content_cl, STARPU_R, v_handle, 0);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_insert_task");
+	ret = starpu_task_insert(&check_content_cl, STARPU_R, v_handle, 0);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 
         ret = starpu_task_wait_for_all();
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait_for_all");
