@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012-2016  Université de Bordeaux
+ * Copyright (C) 2012, 2013, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,12 +19,10 @@
 #include <starpu.h>
 #include "../helper.h"
 
-#ifdef STARPU_USE_CUDA
-#include <cuda.h>
-#endif
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
+/*
+ * Allocate one buffer per worker, doing computations with it, and
+ * eventually reducing it into a single buffer
+ */
 
 #define INIT_VALUE	42
 #define NTASKS		10000
@@ -36,9 +34,9 @@ static uintptr_t per_worker[STARPU_NMAXWORKERS];
 static starpu_data_handle_t per_worker_handle[STARPU_NMAXWORKERS];
 
 /* Create per-worker handles */
-static void initialize_per_worker_handle(void *arg __attribute__((unused)))
+static void initialize_per_worker_handle(void *arg STARPU_ATTRIBUTE_UNUSED)
 {
-	int workerid = starpu_worker_get_id();
+	int workerid = starpu_worker_get_id_check();
 
 	/* Allocate memory on the worker, and initialize it to 0 */
 	switch (starpu_worker_get_type(workerid))
@@ -66,9 +64,16 @@ static void initialize_per_worker_handle(void *arg __attribute__((unused)))
 #endif
 #ifdef STARPU_USE_CUDA
 		case STARPU_CUDA_WORKER:
-			cudaMalloc((void **)&per_worker[workerid], sizeof(variable));
+		{
+		     	cudaError_t status;
+			status = cudaMalloc((void **)&per_worker[workerid], sizeof(variable));
+			if (!per_worker[workerid] || (status != cudaSuccess))
+			{
+				STARPU_CUDA_REPORT_ERROR(status);
+			}
 			cudaMemset((void *)per_worker[workerid], 0, sizeof(variable));
 			break;
+		}
 #endif
 		default:
 			STARPU_ABORT();
@@ -82,10 +87,8 @@ static void initialize_per_worker_handle(void *arg __attribute__((unused)))
  *	Implement reduction method
  */
 
-static void cpu_redux_func(void *descr[], void *cl_arg __attribute__((unused)))
+void cpu_redux_func(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	unsigned *a = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned *b = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[1]);
 
@@ -96,9 +99,7 @@ static void cpu_redux_func(void *descr[], void *cl_arg __attribute__((unused)))
 
 static struct starpu_codelet reduction_codelet =
 {
-	.where = STARPU_CPU,
-	.cpu_funcs = {cpu_redux_func, NULL},
-	.cuda_funcs = {NULL},
+	.cpu_funcs = {cpu_redux_func},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_R},
 	.model = NULL
@@ -108,32 +109,31 @@ static struct starpu_codelet reduction_codelet =
  *	Use per-worker local copy
  */
 
-static void cpu_func_incr(void *descr[], void *cl_arg __attribute__((unused)))
+void cpu_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	*val = *val + 1;
 }
 
 #ifdef STARPU_USE_CUDA
 /* dummy CUDA implementation */
-static void cuda_func_incr(void *descr[], void *cl_arg __attribute__((unused)))
+static void cuda_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
 	unsigned h_val;
-	cudaMemcpy(&h_val, val, sizeof(unsigned), cudaMemcpyDeviceToHost);
+	cudaMemcpyAsync(&h_val, val, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 	h_val++;
-	cudaMemcpy(val, &h_val, sizeof(unsigned), cudaMemcpyHostToDevice);
+	cudaMemcpyAsync(val, &h_val, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
 }
 #endif
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void opencl_func_incr(void *descr[], void *cl_arg __attribute__((unused)))
+static void opencl_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -153,13 +153,13 @@ static void opencl_func_incr(void *descr[], void *cl_arg __attribute__((unused))
 
 static struct starpu_codelet use_data_on_worker_codelet =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_func_incr, NULL},
+	.cpu_funcs = {cpu_func_incr},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cuda_func_incr, NULL},
+	.cuda_funcs = {cuda_func_incr},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {opencl_func_incr, NULL},
+	.opencl_funcs = {opencl_func_incr},
 #endif
 	.nbuffers = 1,
 	.modes = {STARPU_RW},
@@ -171,16 +171,21 @@ int main(int argc, char **argv)
 	unsigned worker;
 	unsigned i;
 	int ret;
+	struct starpu_conf conf;
+
+	starpu_conf_init(&conf);
+	conf.nmic = 0;
+	conf.nscc = 0;
 
 	variable = INIT_VALUE;
 
-        ret = starpu_init(NULL);
+        ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	unsigned nworkers = starpu_worker_get_count();
 
-	starpu_variable_data_register(&variable_handle, 0, (uintptr_t)&variable, sizeof(unsigned));
+	starpu_variable_data_register(&variable_handle, STARPU_MAIN_RAM, (uintptr_t)&variable, sizeof(unsigned));
 
 	/* Allocate a per-worker handle on each worker (and initialize it to 0) */
 	starpu_execute_on_each_worker(initialize_per_worker_handle, NULL, STARPU_CPU|STARPU_CUDA|STARPU_OPENCL);
@@ -207,7 +212,7 @@ int main(int argc, char **argv)
 		task->execute_on_a_specific_worker = 1;
 		task->workerid = (unsigned)workerid;
 
-		int ret = starpu_task_submit(task);
+		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
@@ -222,7 +227,7 @@ int main(int argc, char **argv)
 		task->handles[0] = variable_handle;
 		task->handles[1] = per_worker_handle[worker];
 
-		int ret = starpu_task_submit(task);
+		ret = starpu_task_submit(task);
 		if (ret == -ENODEV) goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
@@ -248,9 +253,6 @@ int main(int argc, char **argv)
 			clReleaseMemObject((void*)per_worker[worker]);
 			break;
 #endif /* !STARPU_USE_OPENCL */
-#ifdef STARPU_USE_GORDON
-		case STARPU_GORDON_WORKER: /* Not supported */	
-#endif /* ! STARPU_USE_GORDON */
 		default:
 			STARPU_ABORT();
 		}
@@ -261,7 +263,10 @@ int main(int argc, char **argv)
 	if (variable == INIT_VALUE + NTASKS)
 		ret = EXIT_SUCCESS;
 	else
+	{
+		FPRINTF(stderr, "%u != %u + %u\n", variable, INIT_VALUE, NTASKS);
 		ret = EXIT_FAILURE;
+	}
 	STARPU_RETURN(ret);
 
 enodev:
