@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2014, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,12 +20,13 @@
 #include <unistd.h>
 #include <errno.h>
 #include <starpu.h>
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Test using a scratch data, using it just for temporary storage
+ */
+
 #ifdef STARPU_QUICK_CHECK
 #  define NLOOPS		8
 #  define VECTORSIZE		128
@@ -40,13 +41,13 @@ starpu_data_handle_t A_handle, B_handle;
 //static unsigned var = 0;
 
 #ifdef STARPU_USE_CUDA
-extern void cuda_f(void *descr[], __attribute__ ((unused)) void *_args);
+extern void cuda_f(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
 #endif
 #ifdef STARPU_USE_OPENCL
 extern void opencl_f(void *buffers[], void *args);
 #endif
 
-static void cpu_f(void *descr[], __attribute__ ((unused)) void *_args)
+void cpu_f(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -67,13 +68,16 @@ static void cpu_f(void *descr[], __attribute__ ((unused)) void *_args)
 
 static struct starpu_codelet cl_f =
 {
-	.cpu_funcs = {cpu_f, NULL},
+	.cpu_funcs = {cpu_f},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cuda_f, NULL},
+	.cuda_funcs = {cuda_f},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {opencl_f, NULL},
+	.opencl_funcs = {opencl_f},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
+	.cpu_funcs_name = {"cpu_f"},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_SCRATCH}
 };
@@ -86,7 +90,7 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -97,7 +101,7 @@ int main(int argc, char **argv)
 #endif
 	A = (unsigned *) calloc(VECTORSIZE, sizeof(unsigned));
 
-	starpu_vector_data_register(&A_handle, 0, (uintptr_t)A, VECTORSIZE, sizeof(unsigned));
+	starpu_vector_data_register(&A_handle, STARPU_MAIN_RAM, (uintptr_t)A, VECTORSIZE, sizeof(unsigned));
 	starpu_vector_data_register(&B_handle, -1, (uintptr_t)NULL, VECTORSIZE, sizeof(unsigned));
 
 	unsigned loop;
@@ -131,7 +135,7 @@ int main(int argc, char **argv)
 	{
 		if (A[i] != NLOOPS)
 		{
-			FPRINTF(stderr, "Error: Incorrect value A[%d] = %u != %d\n", i, A[i], NLOOPS);
+			FPRINTF(stderr, "Error: Incorrect value A[%u] = %u != %d\n", i, A[i], NLOOPS);
 			ret = EXIT_FAILURE;
 			break;
 		}
