@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012-2013  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,10 @@
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Mix submitting tasks and synchronously acquiring the corresponding data.
+ */
+
 #define NBUFFERS_DEF	64
 #define NITER_DEF	128
 #define VECTORSIZE_DEF	1024
@@ -34,7 +38,7 @@ float *buffer[NBUFFERS_DEF];
 
 starpu_data_handle_t v_handle[NBUFFERS_DEF];
 
-static void dummy_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void dummy_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 }
 
@@ -48,9 +52,11 @@ static struct starpu_codelet cl =
 #ifdef STARPU_USE_OPENCL
         .opencl_funcs = {dummy_codelet},
 #endif
+	.cpu_funcs_name = {"dummy_codelet"},
 	.nbuffers = 1
 };
 
+static
 int use_handle(starpu_data_handle_t handle)
 {
 	int ret;
@@ -74,7 +80,7 @@ int main(int argc, char **argv)
 	vectorsize /= 8;
 #endif
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -84,7 +90,7 @@ int main(int argc, char **argv)
 	{
 		ret = starpu_malloc((void **)&buffer[b], vectorsize);
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
-		starpu_vector_data_register(&v_handle[b], 0,
+		starpu_vector_data_register(&v_handle[b], STARPU_MAIN_RAM,
 				(uintptr_t)buffer[b], vectorsize, sizeof(char));
 	}
 
