@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2013  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010-2012, 2014, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +23,10 @@
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Test initializing a buffer with a task, then printing it with another task
+ */
+
 #define VECTORSIZE	1024
 
 starpu_data_handle_t v_handle;
@@ -35,12 +39,11 @@ static void opencl_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_ar
 	cl_mem buf = (cl_mem)STARPU_VECTOR_GET_DEV_HANDLE(descr[0]);
         char ptr = 42;
         cl_command_queue queue;
-        int id = starpu_worker_get_id();
+        int id = starpu_worker_get_id_check();
         int devid = starpu_worker_get_devid(id);
 
         starpu_opencl_get_queue(devid, &queue);
         clEnqueueWriteBuffer(queue, buf, CL_TRUE, 0, sizeof(char), &ptr, 0, NULL, NULL);
-	clFinish(queue);
 }
 #endif
 
@@ -52,18 +55,17 @@ static void cuda_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
 
 	cudaMemsetAsync(buf, 42, 1, starpu_cuda_get_local_stream());
-	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 }
 #endif
 
-static void cpu_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
 
 	*buf = 42;
 }
 
-static void display_var(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void display_var(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -80,10 +82,13 @@ static struct starpu_codelet cl =
 	.cpu_funcs = {cpu_codelet_null},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {cuda_codelet_null},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {opencl_codelet_null},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 1,
 	.modes = {STARPU_W}
 };
@@ -91,6 +96,7 @@ static struct starpu_codelet cl =
 static struct starpu_codelet display_cl =
 {
 	.cpu_funcs = {display_var},
+	.cpu_funcs_name = {"display_var"},
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
@@ -100,7 +106,7 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
