@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2013, 2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2013, 2015-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,11 @@
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Check that we catch calling tag_wait, i.e. a blocking call, from the
+ * codelet function, which is invalid. This test is thus expected to fail.
+ */
+
 #ifdef STARPU_NO_ASSERT
 int main(int argc, char **argv)
 {
@@ -29,9 +34,9 @@ int main(int argc, char **argv)
 #define TAG	0x42
 
 static starpu_data_handle_t handle;
-static unsigned data = 42;
+static unsigned *data;
 
-static void wrong_func(void *descr[], void *arg)
+void wrong_func(void *descr[], void *arg)
 {
 	/* The function is expected to fail. This is indicated in tests/Makefile.am */
 	/* try to fetch data in the RAM while we are in a codelet, such a
@@ -64,12 +69,15 @@ int main(int argc, char **argv)
 	if (RUNNING_ON_VALGRIND)
 		return STARPU_TEST_SKIPPED;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+	starpu_malloc((void**)&data, sizeof(*data));
+	*data = 42;
+
 	/* register a piece of data */
-	starpu_vector_data_register(&handle, 0, (uintptr_t)&data,
+	starpu_vector_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)data,
 						1, sizeof(unsigned));
 
 	struct starpu_task *task = starpu_task_create();
@@ -101,6 +109,9 @@ int main(int argc, char **argv)
 	ret = starpu_task_wait(task);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_wait");
 	starpu_data_unregister(handle);
+
+	starpu_free(data);
+
 	starpu_shutdown();
 
 	return EXIT_SUCCESS;
