@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2012, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,10 @@
 #include "../helper.h"
 #include <common/config.h>
 
+/*
+ * Check that we detect that with only a CPU we can't submit a GPU-only task
+ */
+
 #if !defined(STARPU_HAVE_UNSETENV) || !defined(STARPU_USE_CPU)
 #warning unsetenv is not defined or no cpu are available. Skipping test
 int main(int argc, char **argv)
@@ -27,14 +31,14 @@ int main(int argc, char **argv)
 }
 #else
 
-static void dummy_func(void *descr[], void *arg)
+void dummy_func(void *descr[], void *arg)
 {
 }
 
 static struct starpu_codelet gpu_only_cl =
 {
-	.cuda_funcs = {dummy_func, NULL},
-	.opencl_funcs = {dummy_func, NULL},
+	.cuda_funcs = {dummy_func},
+	.opencl_funcs = {dummy_func},
 	.model = NULL,
 	.nbuffers = 0
 };
@@ -48,8 +52,10 @@ int main(int argc, char **argv)
 	unsetenv("STARPU_NOPENCL");
 	unsetenv("STARPU_NCPUS");
 	struct starpu_conf conf;
-	memset(&conf, 0, sizeof(conf));
+	starpu_conf_init(&conf);
 	conf.ncpus = 1;
+	conf.nopencl = 0;
+	conf.ncuda = 0;
 
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
@@ -58,7 +64,7 @@ int main(int argc, char **argv)
 	struct starpu_task *task = starpu_task_create();
 	task->cl = &gpu_only_cl;
 
-	/* Only a CUDA device could execute that task ! */
+	/* Only a GPU device could execute that task ! */
 	ret = starpu_task_submit(task);
 	STARPU_ASSERT(ret == -ENODEV);
 
