@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2012  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2012-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,10 @@
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Test that starpu_initialize returns ENODEV when no worker is available
+ */
+
 #if !defined(STARPU_HAVE_UNSETENV)
 #warning unsetenv is not defined. Skipping test
 int main(int argc, char **argv)
@@ -32,8 +36,11 @@ int main(int argc, char **argv)
 static void unset_env_variables(void)
 {
 	(void) unsetenv("STARPU_NCPUS");
+	(void) unsetenv("STARPU_NCPU");
 	(void) unsetenv("STARPU_NCUDA");
 	(void) unsetenv("STARPU_NOPENCL");
+	(void) unsetenv("STARPU_NMIC");
+	(void) unsetenv("STARPU_NSCC");
 }
 
 int main(int argc, char **argv)
@@ -48,9 +55,11 @@ int main(int argc, char **argv)
 	conf.ncpus = 0;
 	conf.ncuda = 0;
 	conf.nopencl = 0;
+	conf.nmic = 0;
+	conf.nscc = 0;
 
 	/* starpu_init should return -ENODEV */
-	ret = starpu_init(&conf);
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV)
 	     return EXIT_SUCCESS;
 	else
