@@ -1,8 +1,8 @@
 
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,14 +16,21 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <pthread.h>
 #include <stdio.h>
 #include <unistd.h>
 
 #include <starpu.h>
 #include "../helper.h"
 
-#define NLOOPS	128
+/*
+ * Test that we can declare deps from the callback of the task
+ */
+
+#ifdef STARPU_QUICK_CHECK
+  #define NLOOPS	4
+#else
+  #define NLOOPS	128
+#endif
 
 static void callback(void *arg)
 {
@@ -38,15 +45,16 @@ static void callback(void *arg)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 }
 
-static void dummy_func(void *descr[] __attribute__ ((unused)), void *arg __attribute__ ((unused)))
+void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
 {
 }
 
 static struct starpu_codelet dummy_codelet = 
 {
-	.cpu_funcs = {dummy_func, NULL},
-	.cuda_funcs = {dummy_func, NULL},
-	.opencl_funcs = {dummy_func, NULL},
+	.cpu_funcs = {dummy_func},
+	.cuda_funcs = {dummy_func},
+	.opencl_funcs = {dummy_func},
+	.cpu_funcs_name = {"dummy_func"},
 	.model = NULL,
 	.nbuffers = 0
 };
@@ -66,7 +74,7 @@ int main(int argc, char **argv)
 	int ret;
 	unsigned loop;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
