@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010, 2012-2013, 2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2012, 2015-2016  Université de Bordeaux
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +23,13 @@
 #include "../helper.h"
 #include <common/thread.h>
 
+/*
+ * Test binding tasks on specific workers
+ */
+
 #ifdef STARPU_QUICK_CHECK
+  #define N 10
+#elif !defined(STARPU_LONG_CHECK)
   #define N 100
 #else
   #define N 1000
@@ -43,8 +49,8 @@ static unsigned *v;
 
 static void callback(void *arg)
 {
-	ANNOTATE_HAPPENS_BEFORE(&cnt);
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
+	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
 	if (res == 0)
 	{
@@ -56,9 +62,7 @@ static void callback(void *arg)
 	}
 }
 
-
-
-static void codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 //	int id = starpu_worker_get_id();
 //	FPRINTF(stderr, "worker #%d\n", id);
@@ -69,6 +73,7 @@ static struct starpu_codelet cl_r =
 	.cpu_funcs = {codelet_null},
 	.cuda_funcs = {codelet_null},
         .opencl_funcs = {codelet_null},
+	.cpu_funcs_name = {"codelet_null"},
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
@@ -78,6 +83,7 @@ static struct starpu_codelet cl_w =
 	.cpu_funcs = {codelet_null},
 	.cuda_funcs = {codelet_null},
         .opencl_funcs = {codelet_null},
+	.cpu_funcs_name = {"codelet_null"},
 	.nbuffers = 1,
 	.modes = {STARPU_W}
 };
@@ -87,6 +93,7 @@ static struct starpu_codelet cl_rw =
 	.cpu_funcs = {codelet_null},
 	.cuda_funcs = {codelet_null},
         .opencl_funcs = {codelet_null},
+	.cpu_funcs_name = {"codelet_null"},
 	.nbuffers = 1,
 	.modes = {STARPU_RW}
 };
@@ -111,14 +118,14 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	ret = starpu_malloc((void **)&v, VECTORSIZE*sizeof(unsigned));
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
 
-	starpu_vector_data_register(&v_handle, 0, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
+	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
 
 	unsigned nworker = starpu_worker_get_count();
 
