@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Institut National de Recherche en Informatique et Automatique
- * Copyright (C) 2010-2011, 2014  Université de Bordeaux 1
+ * Copyright (C) 2010  INRIA
+ * Copyright (C) 2010-2011, 2014, 2016  Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,23 +14,33 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
+
 #include <stdio.h>
 #include <starpu.h>
 #include "../helper.h"
 
+/*
+ * Try calling starpu_initialize from different threads in parallel
+ */
+
 #define NUM_THREADS 5
 
+int *glob_argc;
+char ***glob_argv;
+
+static
 void *launch_starpu(void *unused)
 {
 	int ret;
 	(void) unused;
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, glob_argc, glob_argv);
 	if (ret == -ENODEV)
 		exit(STARPU_TEST_SKIPPED);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 	return NULL;
 }
 
+static
 void *shutdown_starpu(void *unused)
 {
 	(void) unused;
@@ -45,6 +55,9 @@ int main(int argc, char **argv)
 	double start;
 	double end;
 
+	glob_argc = &argc;
+	glob_argv = &argv;
+
 	starpu_pthread_t threads[NUM_THREADS];
 
 	start = starpu_timing_now();
