@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
+ * Copyright (C) 2010, 2016  Université de Bordeaux
+ * Copyright (C) 2017        CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,9 +18,13 @@
 #include <starpu.h>
 #include <debug/starpu_debug_helpers.h>
 
+/*
+ * Display the sizes of various StarPU data structures
+ */
+
 int main(int argc, char **argv)
 {
-	_starpu_debug_display_structures_size();
+	_starpu_debug_display_structures_size(stderr);
 
-	return 0;
+	return EXIT_SUCCESS;
 }
