@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2013, 2015  Université de Bordeaux
+ * Copyright (C) 2009-2012, 2015-2016  Université de Bordeaux
  * Copyright (C) 2010  Mehdi Juhoor <mjuhoor@gmail.com>
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012, 2013  CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,6 +25,10 @@
 #include "../helper.h"
 #include <common/thread.h>
 
+/*
+ * Check that working on a partitioned vector gets overlapping of prefetches etc.
+ */
+
 #ifdef STARPU_QUICK_CHECK
 #define NTASKS	100
 #else
@@ -43,8 +47,8 @@ static unsigned cnt = NTASKS;
 
 static void callback(void *arg)
 {
-	ANNOTATE_HAPPENS_BEFORE(&cnt);
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
+	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
 	if (res == 0)
 	{
@@ -56,7 +60,7 @@ static void callback(void *arg)
 	}
 }
 
-static void codelet_sleep(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void codelet_sleep(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -74,6 +78,7 @@ static struct starpu_codelet cl =
 	.cpu_funcs = {codelet_sleep},
 	.cuda_funcs = {codelet_sleep},
         .opencl_funcs = {codelet_sleep},
+	.cpu_funcs_name = {"codelet_sleep"},
 	.nbuffers = 1,
 	.modes = {STARPU_R},
 	.model =  &model
@@ -87,7 +92,7 @@ int main(int argc, char **argv)
 	starpu_data_handle_t handle;
 	float *buffer;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -95,7 +100,7 @@ int main(int argc, char **argv)
 	starpu_malloc((void **)&buffer, NTASKS*VECTORSIZE*sizeof(char));
 
 	/* declare data to StarPU */
-	starpu_vector_data_register(&handle, 0, (uintptr_t)buffer,
+	starpu_vector_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)buffer,
 					NTASKS*VECTORSIZE, sizeof(char));
 
 	struct starpu_data_filter f =
@@ -131,7 +136,7 @@ int main(int argc, char **argv)
 		STARPU_PTHREAD_COND_WAIT(&cond, &mutex);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 
-	starpu_data_unpartition(handle, 0);
+	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
 	starpu_data_unregister(handle);
 	starpu_free(buffer);
 	starpu_shutdown();
