@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2015  Université de Bordeaux
+ * Copyright (C) 2011-2016  Université de Bordeaux
  * Copyright (C) 2011  Télécom-SudParis
- * Copyright (C) 2012 inria
+ * Copyright (C) 2012 INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,10 @@
 #include <starpu_scheduler.h>
 #include "../helper.h"
 
+/*
+ * Benchmark memset with a linear regression
+ */
+
 #define START 1024
 #ifdef STARPU_QUICK_CHECK
 #define END 1048576
@@ -37,7 +41,6 @@ static void memset_cuda(void *descr[], void *arg)
 	unsigned n = STARPU_VECTOR_GET_NX(descr[0]);
 
 	cudaMemsetAsync(ptr, 42, n * sizeof(*ptr), starpu_cuda_get_local_stream());
-	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 }
 #endif
 
@@ -45,7 +48,7 @@ static void memset_cuda(void *descr[], void *arg)
 extern void memset_opencl(void *buffers[], void *args);
 #endif
 
-static void memset_cpu(void *descr[], void *arg)
+void memset_cpu(void *descr[], void *arg)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -71,11 +74,14 @@ static struct starpu_codelet memset_cl =
 {
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {memset_cuda},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {memset_opencl},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 	.cpu_funcs = {memset_cpu},
+	.cpu_funcs_name = {"memset_cpu"},
 	.model = &model,
 	.nbuffers = 1,
 	.modes = {STARPU_W}
@@ -85,11 +91,14 @@ static struct starpu_codelet nl_memset_cl =
 {
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {memset_cuda},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {memset_opencl},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
 	.cpu_funcs = {memset_cpu},
+	.cpu_funcs_name = {"memset_cpu"},
 	.model = &nl_model,
 	.nbuffers = 1,
 	.modes = {STARPU_W}
@@ -129,8 +138,8 @@ static void show_task_perfs(int size, struct starpu_task *task)
 		unsigned nimpl;
 		for (nimpl = 0; nimpl < STARPU_MAXIMPLEMENTATIONS; nimpl++)
 		{
-			FPRINTF(stdout, "Expected time for %d on %s (impl %d):\t%f\n",
-				size, name, nimpl, starpu_task_expected_length(task, starpu_worker_get_perf_archtype(workerid), nimpl));
+			FPRINTF(stdout, "Expected time for %d on %s (impl %u):\t%f\n",
+				size, name, nimpl, starpu_task_expected_length(task, starpu_worker_get_perf_archtype(workerid, task->sched_ctx), nimpl));
 		}
 	}
 }
@@ -150,7 +159,7 @@ int main(int argc, char **argv)
 	conf.sched_policy_name = "eager";
 	conf.calibrate = 2;
 
-	ret = starpu_init(&conf);
+	ret = starpu_initialize(&conf, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
