@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013 Université Bordeaux 1
+ * Copyright (C) 2013, 2015-2016 Université Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,18 +19,20 @@
 #include <starpu_scheduler.h>
 #include "../helper.h"
 
-#ifdef STARPU_QUICK_CHECK
-#define NTASKS 10
-#else
-#define NTASKS 1000
-#endif
-
 /*
  * Task1 must be executed before task0, even if task0 is submitted first.
  * Applies to : all schedulers.
  */
 
-static void
+#ifdef STARPU_QUICK_CHECK
+#define NTASKS 10
+#elif !defined(STARPU_LONG_CHECK)
+#define NTASKS 100
+#else
+#define NTASKS 1000
+#endif
+
+void
 A(void *buffers[], void *args)
 {
 	(void) buffers;
@@ -39,7 +41,7 @@ A(void *buffers[], void *args)
 	usleep(1000);
 }
 
-static void
+void
 B(void *buffers[], void *args)
 {
 	(void) buffers;
@@ -65,24 +67,30 @@ run(struct starpu_sched_policy *policy)
 	struct starpu_codelet clA =
 	{
 		.cpu_funcs = {A},
+		.cpu_funcs_name = {"A"},
 		.nbuffers = 0
 	};
 
 	struct starpu_codelet clB =
 	{
 		.cpu_funcs = {B},
+		.cpu_funcs_name = {"B"},
 		.nbuffers = 0
 	};
 
         starpu_srand48(0);
 
-	for (i = 0; i < NTASKS; i++) {
+	for (i = 0; i < NTASKS; i++)
+	{
 		struct starpu_task *task = starpu_task_create();
 
-		if (((int)(starpu_drand48()*2))%2) {
+		if (((int)(starpu_drand48()*2))%2)
+		{
 			task->cl = &clA;
 			task->priority=STARPU_MIN_PRIO;
-		} else {
+		}
+		else
+		{
 			task->cl = &clB;
 			task->priority=STARPU_MAX_PRIO;
 		}
@@ -109,6 +117,10 @@ main(void)
 	struct starpu_sched_policy **policies;
 	struct starpu_sched_policy **policy;
 
+#ifdef STARPU_HAVE_UNSETENV
+	unsetenv("STARPU_SCHED");
+#endif
+
 	policies = starpu_sched_get_predefined_policies();
 	for(policy=policies ; *policy!=NULL ; policy++)
 	{
