@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012 Inria
+ * Copyright (C) 2012 INRIA
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,8 +24,7 @@
  * Applies to : all schedulers.
  */
 
-static void
-dummy(void *buffers[], void *args)
+void dummy(void *buffers[], void *args)
 {
 	(void) buffers;
 	(void) args;
@@ -47,6 +46,7 @@ run(struct starpu_sched_policy *policy)
 	struct starpu_codelet cl =
 	{
 		.cpu_funcs = {dummy},
+		.cpu_funcs_name = {"dummy"},
 		.nbuffers = 0
 	};
 
@@ -90,6 +90,10 @@ main(void)
 	struct starpu_sched_policy **policies;
 	struct starpu_sched_policy **policy;
 
+#ifdef STARPU_HAVE_UNSETENV
+	unsetenv("STARPU_SCHED");
+#endif
+
 	policies = starpu_sched_get_predefined_policies();
 	for(policy=policies ; *policy!=NULL ; policy++)
 	{
